/*****************************************************************************
 * vlvc.c: VLC's core module for VLVC
 *****************************************************************************
 * Copyright (C) 2005 - 2008 the VLVC team
 * $Id: vlvc.c
 *
 * Authors: Julien ARNAUD <arnaud_j@epitech.net>
 *          Julien CUPILLARD <cupill_j@epitech.net>
 *          Thomas DELON <delon_t@epitech.net>
 *          Arnaud DEREDEC <derede_a@epitech.net>
 *          Florian GUSTIN <gustin_f@epitech.net>
 *          Sylvestre HO TAM CHAY <ho-tam_s@epitech.net>
 *          Romain LAVOCAT <lavoca_r@epitech.net>
 *          William LIENARD <lienar_w@epitech.net>
 *          Guillaume MARTIN <martin_u@epitech.net>
 *          Florent NGUYEN <nguyen_c@epitech.net>
 *          Julien PASSERI <passer_j@epitech.net>
 *          Mathavarajan SUGUMARAN <suguma_m@epitech.net>
 *          David YU <yu_d@epitech.net>
 *
 * This software has been written in EPITECH <http://www.epitech.net>
 * EPITECH is computer science school in Paris - FRANCE -
 * under the direction of Flavien Astraud <http://www.epita.fr/~flav>,
 * Jerome Landrieu and Launiau Severin.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/

#include <vlc/intf.h>

#include "vlc_vlm.h"
#include "vlc_vlvc.h"
#include "network.h"

#include "vlc_vlvc_error.h"

#include <wchar.h>

/*****************************************************************************
 * Local prototypes.
 *****************************************************************************/
static int HandleVlmUserControl( vlvc_t *p_vlvc, int i_user_id,
                                 char *psz_vlm_command_type, char *psz_mrl );
static int CreateVlmUserControl( vlvc_t *p_vlvc, int i_user_id );
static int PlayVlmUserControl( vlvc_t *p_vlvc, int i_user_id );
static int StopVlmUserControl( vlvc_t *p_vlvc, int i_user_id );
static int DeleteVlmUserControl( vlvc_t *p_vlvc, int i_user_id );
static int SetupInputVlmUserControl( vlvc_t *p_vlvc, int i_user_id, char *psz_mrl );
static int SetupOptionVlmUserControl( vlvc_t *p_vlvc, int i_user_id, char *psz_mrl );
static int SetupOutputVlmUserControl( vlvc_t *p_vlvc, int i_user_id, char *psz_mrl );
static void GuiDisplayMessage( vlvc_t *p_vlvc, vlvc_message_header_t *p_message_header,
                               const char *psz_message );
static int HandlePassword(vlvc_t *p_vlvc, vlvc_user_t *p_current_user,
                          int i_user_index );
static int HandleVersion(vlvc_t *p_vlvc, vlvc_user_t *p_current_user,
                         int i_user_index );

static int GenerateMulticastInfo( vlvc_t *p_vlvc );
static int SendMulticastInfo( vlvc_t *p_vlvc, int i_user_index );
static int ReceiveMulticastInfo( vlvc_t *p_vlvc );
static int ServerTryMulticast( vlvc_t *p_vlvc, int i_user_index );
static int ClientTryMulticast( vlvc_t *p_vlvc );
static int ReceiveMulticastReply( vlvc_t *p_vlvc, int i_user_index );

void dbg(vlvc_t *p_vlvc, char *psz_message )
{
    vlvc_message_header_t hdr;
    hdr.c_user_src_id = 0;
    hdr.c_type = 0;
    hdr.i_size = strlen(psz_message);
    GuiDisplayMessage( p_vlvc, &hdr, psz_message );
}

/*****************************************************************************
 * Name: CopyAndAddBackslashes
 *
 * Parameters:
 *             char *psz_src    : the source string
 *             char *psz_dest   : the destination string
 *             int  i_dest_size : the size of the destination string
 *
 * Returns:
 *             void
 *
 * Description:
 *             Copies the source in the destination, and adds backslashes
 *             before spaces.
 *****************************************************************************/
static void CopyAndAddBackslashes( char *psz_dest, char *psz_src, int i_dest_size )
{
    char *psz_dest_end = psz_dest + i_dest_size - 1;

    memset( psz_dest, 0, i_dest_size );
    while( *psz_src && psz_dest != psz_dest_end )
    {
        if( *psz_src == ' ' )
        {
            *psz_dest++ = '\\';
            if( psz_dest == psz_dest_end )
            {
                /* If the backslash just added is the last
                   character of the string */
                *( psz_dest - 1 ) = '\0';
                break;
            }

        }
        *psz_dest++ = *psz_src++;
    }
}

/*****************************************************************************
 * Name: __vlvc_New
 *
 * Parameters:
 *             vlc_object_t * p_this : a pointer to the common object structure
 *
 * Returns:
 *             vlvc_t * : pointer to an instance of the vlvc object
 *
 * Description:
 *             Retreives or creates the vlvc object
 *****************************************************************************/
vlvc_t *__vlvc_New( vlc_object_t *p_this )
{
    vlc_value_t lockval;
    vlvc_t      *p_vlvc = NULL;
    int         i_user_index;

    //var_Create( p_this->p_libvlc, VLVC_MUTEX, VLC_VAR_MUTEX );
    var_Get( p_this->p_libvlc, VLVC_MUTEX, &lockval );
    vlc_mutex_lock( lockval.p_address );

    if( !( p_vlvc = vlc_object_find( p_this, VLC_OBJECT_VLVC, FIND_ANYWHERE ) ) )
    {
        if( ( p_vlvc = vlc_object_create( p_this, VLC_OBJECT_VLVC ) ) == NULL )
        {
            vlc_mutex_unlock( lockval.p_address );
            return( NULL );
        }

        vlc_mutex_init( p_this->p_vlc, &p_vlvc->common_lock );
        vlc_mutex_init( p_this->p_vlc, &p_vlvc->server_lock );
        vlc_mutex_init( p_this->p_vlc, &p_vlvc->client_lock );
        vlc_mutex_init( p_this->p_vlc, &p_vlvc->accept_lock );
        vlc_mutex_init( p_this->p_vlc, &p_vlvc->command_lock );
        vlc_mutex_init( p_this->p_vlc, &p_vlvc->gui_lock );
        vlc_mutex_init( p_this->p_vlc, &p_vlvc->network_lock );

        p_vlvc->i_nb_users = 0;
        p_vlvc->i_pending_users = 0;
        p_vlvc->i_self_index = VLVC_INVALID_CLIENT_ID;
        p_vlvc->b_initialized = VLC_FALSE;
        p_vlvc->b_accept = VLC_FALSE;
        p_vlvc->b_started = VLC_FALSE;
        p_vlvc->i_last_error = 0;
        p_vlvc->b_GUIPresent = VLC_FALSE;

        memset( p_vlvc->p_pending_fd, VLVC_INVALID_FD, sizeof( p_vlvc->p_pending_fd ) );

        for( i_user_index = 0; i_user_index < VLVC_MAX_USERS; i_user_index++ )
        {
            p_vlvc->p_users[i_user_index].b_free = VLC_TRUE;
            p_vlvc->p_meeting_auth_users[i_user_index] = VLVC_AUTH_FREE;
        }

        p_vlvc->i_amphi_auth_user = VLVC_AUTH_FREE;

        p_vlvc->p_vlm = vlm_New( p_this );

        vlc_object_yield( p_vlvc );
        vlc_object_attach( p_vlvc, p_this->p_vlc );
    }

    vlc_mutex_unlock( lockval.p_address );

    return( p_vlvc );
}

/*****************************************************************************
 * Name: vlvc_Delete
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             void
 *
 * Description:
 *             frees and releases if needed the vlvc "object"
 *
 * Comments:
 * As done in other modules, we are supposed to call to vlc_thread_join to clean
 * our thread. But in our case, as we create many other threads, different
 * issues occur when we call it.
 * There is no problem not to call vlc_thread_join as our threads exit properly.
 *****************************************************************************/
void vlvc_Delete( vlvc_t *p_vlvc )
{
    vlc_value_t      lockval;
    int              i_current_user;

    if( p_vlvc == NULL )
        return;
    var_Get( p_vlvc->p_libvlc, VLVC_MUTEX, &lockval );
    vlc_mutex_lock( lockval.p_address );

    vlc_object_release( p_vlvc );

    if( p_vlvc->i_refcount > 0 )
    {
        vlc_mutex_unlock( lockval.p_address );
        return;
    }

    for( i_current_user = 0; i_current_user < p_vlvc->i_nb_users; i_current_user++ )
    {
        if( p_vlvc->p_users[i_current_user].b_free == VLC_FALSE )
        {
            StopVlmUserControl( p_vlvc, i_current_user );
            DeleteVlmUserControl( p_vlvc, i_current_user );
            //net_Close( p_vlvc->p_users[i_current_user].i_fd );
        }
    }

    if( p_vlvc->p_parameters->i_operating_mode == VLVC_SERVER_MODE )
    {
        net_ListenClose( p_vlvc->p_listen_fd );
    }

    p_vlvc->b_die = VLC_TRUE;

    if( p_vlvc->b_started == VLC_TRUE )
        vlc_thread_join( p_vlvc );

    vlc_mutex_destroy( &p_vlvc->common_lock );
    vlc_mutex_destroy( &p_vlvc->server_lock );
    vlc_mutex_destroy( &p_vlvc->client_lock );
    vlc_mutex_destroy( &p_vlvc->accept_lock );
    vlc_mutex_destroy( &p_vlvc->command_lock );
    vlc_mutex_destroy( &p_vlvc->gui_lock );
    vlc_mutex_destroy( &p_vlvc->network_lock );

    p_vlvc->b_started = VLC_FALSE;

    vlm_Delete( p_vlvc->p_vlm );

    vlc_object_detach( p_vlvc );
    vlc_object_destroy( p_vlvc );
    vlc_mutex_unlock( lockval.p_address );
}

/*****************************************************************************
 * Name: NetWrite
 *
 * Parameters:
 *             vlvc_t *p_vlvc    : a pointer to the vlvc object
 *             int    i_fd       : the fd to whom we want to send data
 *             void   *p_content : data to be sent
 *             int    i_size     : size of the data to be sent
 *
 * Returns:
 *             int : number of bytes written
 *
 * Description:
 *             Wrapper for net_Write (sends data on the network)
 *****************************************************************************/
static int NetWrite(vlvc_t *p_vlvc, int i_fd, const void *p_content, int i_size)
{
    int i_ret = VLVC_WRITE_ERROR;

    if( i_fd != VLVC_INVALID_FD )
    {
        int             i_select_ret;
        struct timeval  timeout;
        fd_set          fds_w;

        vlc_mutex_lock( &p_vlvc->network_lock );

        FD_ZERO( &fds_w );
        FD_SET( i_fd, &fds_w );
        timeout.tv_sec = 0;
        timeout.tv_usec = 100000;
        i_select_ret = select( i_fd + 1, NULL, &fds_w, NULL, &timeout );
        if( i_select_ret > 0 )
            i_ret = net_Write( p_vlvc, i_fd, NULL, p_content, i_size );

        vlc_mutex_unlock( &p_vlvc->network_lock );

    }
    return( i_ret );
}

/*****************************************************************************
 * Name: ServerNetWrite
 *
 * Parameters:
 *             vlvc_t *p_vlvc    : a pointer to the vlvc object
 *             int    i_user_id  : the user id we want to send data to
 *             void   *p_content : data to be sent
 *             int    i_size     : size of the data to be sent
 *
 * Returns:
 *             int : number of bytes written
 *
 * Description:
 *             Used by the server to send data
 *****************************************************************************/
static int ServerNetWrite(vlvc_t *p_vlvc, int i_user_id, const void *p_content, int i_size)
{
    return( NetWrite( p_vlvc, p_vlvc->p_users[i_user_id].i_fd,
                      p_content, i_size ) );
}

/*****************************************************************************
 * Name: ClientNetWrite
 *
 * Parameters:
 *             vlvc_t *p_vlvc    : a pointer to the vlvc object
 *             void   *p_content : data to be sent
 *             int    i_size     : size of the data to be sent
 *
 * Returns:
 *             int : number of bytes written
 *
 * Description:
 *             Used by the client to send data
 *****************************************************************************/
static int ClientNetWrite(vlvc_t *p_vlvc, const void *p_content, int i_size)
{
    return( NetWrite( p_vlvc, p_vlvc->i_client_socket,
                      p_content, i_size ) );
}

/*****************************************************************************
 * Name: NetRead
 *
 * Parameters:
 *             vlvc_t *p_vlvc    : a pointer to the vlvc object
 *             int    i_fd       : the file descriptor on which data is to be
 *                                 read
 *             void   *p_content : buffer to store read data
 *             int    i_size     : size of the data to be read
 *
 * Returns:
 *             int : number of bytes read
 *
 * Description:
 *             Wrapper for net_Read (reads data on the network)
 *****************************************************************************/
static int NetRead(vlvc_t *p_vlvc, int i_fd, void *p_content, int i_size)
{
    int i_ret;
    if( i_fd <= 0 )
        return( VLVC_READ_ERROR );
    i_ret = net_Read( p_vlvc, i_fd, NULL, p_content, i_size, VLC_TRUE );
    if( i_ret <= 0 )
        i_ret = VLVC_READ_ERROR;
    return i_ret;
}

/*****************************************************************************
 * Name: ClientNetRead
 *
 * Parameters:
 *             vlvc_t *p_vlvc    : a pointer to the vlvc object
 *             void   *p_content : buffer to store read data
 *             int    i_size     : size of the data to be read
 *
 * Returns:
 *             int : number of bytes written
 *
 * Description:
 *             Used by the client to read data
 *****************************************************************************/
static int ClientNetRead(vlvc_t *p_vlvc, void *p_content, int i_size)
{
    return( NetRead( p_vlvc, p_vlvc->i_client_socket, p_content, i_size) );
}

/*****************************************************************************
 * Name: ServerNetRead
 *
 * Parameters:
 *             vlvc_t *p_vlvc    : a pointer to the vlvc object
 *             int    i_fd       : a file descriptor to read data
 *             void   *p_content : data to be sent
 *             int    i_size     : size of the data to be sent
 *
 * Returns:
 *             int : number of bytes written
 *
 * Description:
 *             used by the server to send data safely
 *****************************************************************************/
static int ServerNetRead(vlvc_t *p_vlvc, int i_fd, void *p_content, int i_size)
{
    return( NetRead( p_vlvc, i_fd, p_content, i_size ) );
}

/*****************************************************************************
 * Name: GuiNbClientChange
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : the userid to whom we want to send data
 *
 * Returns:
 *             void
 *
 * Description:
 *             Wrapper for the GUI callback used to notify a change of the
 *             number of users.
 *****************************************************************************/
static void GuiNbClientChange( vlvc_t *p_vlvc, int i_user_id )
{
    if( p_vlvc->NbClientChange != NULL )
    {
        vlc_mutex_lock( &p_vlvc->gui_lock );
        p_vlvc->NbClientChange( (int) i_user_id, NULL);
        vlc_mutex_unlock( &p_vlvc->gui_lock );
    }
}

/*****************************************************************************
 * Name: GuiAuthUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : the userid to whom we want to send data
 *
 * Returns:
 *             void
 *
 * Description:
 *             Wrapper for the GUI callback used to notify a change of the
 *             user auth
 *****************************************************************************/
static void GuiAuthUser( vlvc_t *p_vlvc, int i_user_id )
{
    if( p_vlvc->AuthUser != NULL )
    {
        vlc_mutex_lock( &p_vlvc->gui_lock );
        p_vlvc->AuthUser( (int) i_user_id, NULL);
        vlc_mutex_unlock( &p_vlvc->gui_lock );
    }
}

/*****************************************************************************
 * Name: GuiUnauthUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : the userid to whom we want to send data
 *
 * Returns:
 *             void
 *
 * Description:
 *             Wrapper for the GUI callback used to notify a change of the
 *             unauth user
 *****************************************************************************/
static void GuiUnauthUser( vlvc_t *p_vlvc, int i_user_id )
{
    if( p_vlvc->UnauthUser != NULL )
    {
        vlc_mutex_lock( &p_vlvc->gui_lock );
        p_vlvc->UnauthUser( (int) i_user_id, NULL);
        vlc_mutex_unlock( &p_vlvc->gui_lock );
    }
}

/*****************************************************************************
 * Name: GuiDisplayMessage
 *
 * Parameters:
 *             vlvc_t     *p_vlvc      : a pointer to the vlvc object
 *             vlvc_message_header_t *p_message_header : a pointer to the
 *                                                       message header
 *             const char *psz_message : a pointer to the message to display
 *
 *
 * Returns:
 *             void
 *
 * Description:
 *             Wrapper for the GUI callback used to display a chat message.
 *****************************************************************************/
static void GuiDisplayMessage( vlvc_t *p_vlvc,
                               vlvc_message_header_t *p_message_header,
                               const char *psz_message )
{
    if( p_vlvc->ChatMessage != NULL )
    {
        vlc_mutex_lock( &p_vlvc->gui_lock );
        p_vlvc->ChatMessage( (int)p_message_header->c_user_src_id,
                             psz_message,
                             (int)p_message_header->c_type, NULL );
        vlc_mutex_unlock( &p_vlvc->gui_lock );
    }
}

/*****************************************************************************
 * Name: GuiMessageBox
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             char   *psz_message : a pointer to the message to display
 *             int    i_critical   : the critical level of the message
 *
 * Returns:
 *             void
 *
 * Description:
 *             Wrapper for the GUI callback used to display a message box.
 *****************************************************************************/
static void GuiMessageBox( vlvc_t *p_vlvc, char *psz_message, int i_critical )
{
    if( p_vlvc->CoreModuleMessage != NULL )
    {
        vlc_mutex_lock( &p_vlvc->gui_lock );
        p_vlvc->CoreModuleMessage( psz_message, i_critical, NULL );
        vlc_mutex_unlock( &p_vlvc->gui_lock );
    }
}

/*****************************************************************************
 * Name: GuiStop
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             void
 *
 * Description:
 *             Wrapper for the GUI callback used to stop it.
 *****************************************************************************/
static void GuiStop( vlvc_t *p_vlvc )
{
    if( p_vlvc->Stop != NULL )
        p_vlvc->Stop( NULL );
}

/*****************************************************************************
 * Name: GetUserId
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : user ID
 *
 * Description:
 *             returns the first available user ID, or
 *             VLVC_NO_MORE_USERS_ALLOWED if there is no free connection left.
 *****************************************************************************/
static int GetUserId( vlvc_t *p_vlvc )
{
    int    i_user_index;

    if( p_vlvc == NULL )
        return( VLVC_EARGS );

    for( i_user_index = 0; i_user_index < VLVC_MAX_USERS; i_user_index++ )
        if( p_vlvc->p_users[i_user_index].b_free == VLC_TRUE )
            break;
    if( i_user_index == VLVC_MAX_USERS )
        return( VLVC_NO_MORE_USERS_ALLOWED );
    return( i_user_index );
}

/*****************************************************************************
 * Name: ServerUserLeave
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             int    i_user_index : ID of the user to delete
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Deletes the specified user and informs the GUI.
 *****************************************************************************/
static int ServerUserLeave( vlvc_t *p_vlvc, int i_user_index )
{
    vlvc_user_t    *current_user = &p_vlvc->p_users[i_user_index];
    int            i_count;

    //    memset( current_user->psz_name, STRING_TERMINATOR, VLVC_MAX_USER_NAME_LENGTH );
    memset( current_user->psz_description, STRING_TERMINATOR, VLVC_MAX_USER_DESC_LENGTH );
    memset( current_user->psz_ip_address, STRING_TERMINATOR, VLVC_MAX_IP_ADDRESS_LENGTH );

    StopVlmUserControl( p_vlvc, i_user_index );
    DeleteVlmUserControl( p_vlvc, i_user_index );

    if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE &&
        p_vlvc->i_amphi_auth_user == i_user_index )
        p_vlvc->i_amphi_auth_user = VLVC_AUTH_FREE;
    else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
    {
        for( i_count = 0; i_count < VLVC_MAX_USERS; i_count++ )
            if( p_vlvc->p_meeting_auth_users[i_count] == i_user_index )
                p_vlvc->p_meeting_auth_users[i_count] = VLVC_AUTH_FREE;
    }

    current_user->i_fd = VLVC_INVALID_FD;
    current_user->b_free = VLC_TRUE;
    current_user->b_multicast = VLC_FALSE;
    p_vlvc->i_nb_users--;

    GuiNbClientChange( p_vlvc, i_user_index );

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: AddOutputDestination
 *
 * Parameters:
 *             vlvc_t *p_vlvc         : a pointer to the vlvc object
 *             int    i_user_src      : the user from which we create the
 *                                      outputs
 *             char   *psz_output_mrl : output buffer to be filled
 *
 * Returns:
 *             int : number of users added
 *
 * Description:
 *             Creates the output MRL for a given user.
 *****************************************************************************/
static int AddOutputDestination( vlvc_t *p_vlvc, int i_user_src, char psz_output_mrl[VLVC_MAX_MRL_LEN] )
{
   int              i_user_dest;
   int              i_nb_user_added = 0;
   int              b_multicast_on = VLC_FALSE;

   printf("\n---------------- AddOutputDestination ------------------\n"); //AFF
   printf("i_user_src = %d\n\n", i_user_src); //AFF
   for( i_user_dest = 0; i_user_dest < p_vlvc->i_nb_users; i_user_dest++ )
   {
       if( ( i_user_dest != i_user_src ) &&
           ( i_user_dest != p_vlvc->i_self_index ) )
       {
           char *psz_mrl = VLVC_SPRINTF_SERVER_NET_DUPLICATE;

           if( p_vlvc->p_parameters->b_standalone == VLC_TRUE && i_nb_user_added == 0 )
               psz_mrl = VLVC_SPRINTF_SERVER_STANDALONE_FIRST_NET_DUPLICATE;

           printf("i_user_dest = %d\n", i_user_dest); //AFF
           if( p_vlvc->p_users[i_user_dest].b_multicast == VLC_TRUE &&
               b_multicast_on == VLC_FALSE )
           {
               printf("== TRUE ==\n"); //AFF
               snprintf( psz_output_mrl + strlen( psz_output_mrl ),
                         VLVC_MAX_MRL_LEN, psz_mrl,
                         p_vlvc->p_parameters->psz_network_protocol,
                         p_vlvc->psz_multicast_group,
                         VLVC_FIRST_CLIENT_PORT_DEFAULT + i_user_src );
               b_multicast_on = VLC_TRUE;
           }
           else if( p_vlvc->p_users[i_user_dest].b_multicast == VLC_FALSE )
           {
               printf("== FALSE ==\n"); //AFF
               snprintf( psz_output_mrl + strlen( psz_output_mrl ),
                         VLVC_MAX_MRL_LEN, psz_mrl,
                         p_vlvc->p_parameters->psz_network_protocol,
                         p_vlvc->p_users[i_user_dest].psz_ip_address,
                         p_vlvc->p_users[i_user_dest].i_first_client_port
                         + i_user_src );
           }
           else
               printf("== ALREADY ON ==\n"); //AFF
           i_nb_user_added++;
       }
   }
   psz_output_mrl[strlen(psz_output_mrl)] = '}';
   printf("----------------------------------------------------------\n\n");//AFF

   return( i_nb_user_added );
}

/*****************************************************************************
 * Name: SetupClientsOutputsChatRoom
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             void
 *
 * Description:
 *             Setup all client output for the chat room mode
 *****************************************************************************/
static void SetupClientsOutputsChatRoom( vlvc_t *p_vlvc )
{
   int                  i_user_src;
   char                 psz_output_mrl[VLVC_MAX_MRL_LEN];
   int                  i_nb_user_added = 0;
   vlvc_param_t         *p_parameters = p_vlvc->p_parameters;

   if( p_parameters->b_standalone == VLC_TRUE && p_vlvc->i_nb_users < 2 )
       return;

   /* patch due to device busy on stoping/playing webcam quickly*/
   if( p_parameters->b_standalone == VLC_FALSE )
       StopVlmUserControl( p_vlvc, p_vlvc->i_self_index );

   for( i_user_src = 0; i_user_src < p_vlvc->i_nb_users; i_user_src++ )
   {
       memset( psz_output_mrl , STRING_TERMINATOR, VLVC_MAX_MRL_LEN );
       if( i_user_src == p_vlvc->i_self_index )
       {
           snprintf( psz_output_mrl, sizeof( psz_output_mrl ),
                     VLVC_SPRINTF_SELF_SERVER_BEGIN_OUTPUT_MRL,
                     p_parameters->psz_video_codec, p_parameters->i_video_bitrate,
                     p_parameters->psz_video_scale,
                     p_parameters->psz_audio_codec, p_parameters->i_audio_bitrate );
       }
       else
       {
           if( p_parameters->b_standalone == VLC_FALSE )
               snprintf( psz_output_mrl, sizeof( psz_output_mrl ),
                          VLVC_SPRINTF_SERVER_BEGIN_OUTPUT_MRL );
       }

       i_nb_user_added += AddOutputDestination( p_vlvc, i_user_src, psz_output_mrl );

       if( i_nb_user_added > 0 )
       {
           /* patch due to device busy on stoping/playing webcam quickly*/
           if( i_user_src == p_vlvc->i_self_index )
                   SetupOutputVlmUserControl( p_vlvc, i_user_src, psz_output_mrl );
           else
           {
               StopVlmUserControl( p_vlvc, i_user_src );
               SetupOutputVlmUserControl( p_vlvc, i_user_src, psz_output_mrl );
               PlayVlmUserControl( p_vlvc, i_user_src );
           }
           msleep( 10000 );
       }
   }
   /* patch due to device busy on stoping/playing webcam quickly*/
   if( p_parameters->b_standalone == VLC_FALSE )
       PlayVlmUserControl( p_vlvc, p_vlvc->i_self_index );
}

/*****************************************************************************
 * Name: SetupClientsOutputsConference
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             void
 *
 * Description:
 *             Setup all client output for the conference mode
 *****************************************************************************/
static void SetupClientsOutputsConference( vlvc_t *p_vlvc )
{
   char                 psz_output_mrl[VLVC_MAX_MRL_LEN];
   vlvc_param_t         *p_parameters = p_vlvc->p_parameters;

   StopVlmUserControl( p_vlvc, p_vlvc->i_self_index );
   snprintf( psz_output_mrl, sizeof( psz_output_mrl ),
             VLVC_SPRINTF_SELF_SERVER_BEGIN_OUTPUT_MRL,
             p_parameters->psz_video_codec, p_parameters->i_video_bitrate,
             p_parameters->psz_video_scale,
             p_parameters->psz_audio_codec, p_parameters->i_audio_bitrate );

   AddOutputDestination( p_vlvc, p_vlvc->i_self_index, psz_output_mrl );
   SetupOutputVlmUserControl( p_vlvc, p_vlvc->i_self_index, psz_output_mrl );
   PlayVlmUserControl( p_vlvc, p_vlvc->i_self_index );
}

/*****************************************************************************
 * Name: NotifyUserJoin
 *
 * Parameters:
 *             vlvc_t *p_vlvc          : a pointer to the vlvc object
 *             char   c_new_user_index : the new user ID
 *
 * Returns:
 *             void
 *
 * Description:
 *             Notify clients when a new user has joined
 *****************************************************************************/
static void NotifyUserJoin( vlvc_t *p_vlvc, int i_new_user_index )
{
    int                                 i_dest_user;
    vlvc_server_user_info_t             user_info;
    char                                c_command = VLVC_NET_COMMAND_ADD_USER;

    user_info.c_user_id = (char)i_new_user_index;
    memcpy( user_info.psz_name, p_vlvc->p_users[i_new_user_index].psz_name,
             sizeof( user_info.psz_name ) );
    memcpy( user_info.psz_description, p_vlvc->p_users[i_new_user_index].psz_description,
             sizeof( user_info.psz_description ) );
    for( i_dest_user = 0; i_dest_user < p_vlvc->i_nb_users; i_dest_user++ )
    {
        if( i_dest_user != i_new_user_index && i_dest_user != p_vlvc->i_self_index )
        {
            ServerNetWrite( p_vlvc, i_dest_user, &c_command, sizeof( c_command ) );
            ServerNetWrite( p_vlvc, i_dest_user, &user_info, sizeof( user_info ) );
            msleep( 10000 ); //a verifer
        }
    }

    for( i_dest_user = 0; i_dest_user < p_vlvc->i_nb_users; i_dest_user++ )
    {
        user_info.c_user_id = (char)i_dest_user;
        memcpy( user_info.psz_name, p_vlvc->p_users[i_dest_user].psz_name,
                 sizeof( user_info.psz_name ) );
        memcpy( user_info.psz_description, p_vlvc->p_users[i_dest_user].psz_description,
                 sizeof( user_info.psz_description ) );

        ServerNetWrite( p_vlvc, i_new_user_index, &c_command, sizeof( c_command ) );
        ServerNetWrite( p_vlvc, i_new_user_index, &user_info, sizeof( user_info ) );
        msleep( 10000 ); //a verifer
    }
}


/*****************************************************************************
 * Name: SendLeaveUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc            : a pointer to the vlvc object
 *             char   c_user_leave_index : the leaving user ID
 *
 * Returns:
 *             void
 *
 * Description:
 *             Used by the server to inform every client someone has left.
 *****************************************************************************/
static void SendLeaveUser( vlvc_t *p_vlvc, char c_user_leave_index )
{
    int                         i_dest_user;
    char                        c_command = VLVC_NET_COMMAND_LEAVE_USER;

    for( i_dest_user = 0; i_dest_user < p_vlvc->i_nb_users; i_dest_user++ )
        if( i_dest_user != c_user_leave_index && i_dest_user != p_vlvc->i_self_index )
        {
            ServerNetWrite( p_vlvc, i_dest_user, &c_command, sizeof( c_command ) );
            ServerNetWrite( p_vlvc, i_dest_user, &c_user_leave_index,
                           sizeof( c_user_leave_index ) );
        }
}

/*****************************************************************************
 * Name: SendWelcome
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             int    i_user_index : the client id to send welcome to
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called when a user sends a hello message, to welcome him
 *****************************************************************************/
static int SendWelcome( vlvc_t *p_vlvc, int i_user_index )
{
    vlvc_welcome_data_t      welcome_data;
    char                     c_command = VLVC_NET_COMMAND_WELCOME;
    int                      i_count;

    welcome_data.c_user_id = (char)i_user_index;
    welcome_data.c_conference_mode = (char)p_vlvc->p_parameters->i_conference_mode;
    welcome_data.c_server_id = (char)p_vlvc->i_self_index;
    memset( welcome_data.psz_server_name, STRING_TERMINATOR,
            sizeof( welcome_data.psz_server_name ) );
    snprintf( welcome_data.psz_server_name,
              VLVC_BUFFER_SIZE( welcome_data.psz_server_name ),
              p_vlvc->p_parameters->psz_username );
    welcome_data.i_first_stream_port = p_vlvc->p_parameters->i_first_client_port;
    if( p_vlvc->p_users[i_user_index].b_multicast == VLC_TRUE &&
        p_vlvc->p_parameters->b_multicast == VLC_TRUE )
        welcome_data.b_multicast = VLC_TRUE;
    else
        welcome_data.b_multicast = VLC_FALSE;

    welcome_data.i_amphi_auth_user = p_vlvc->i_amphi_auth_user;
    for( i_count = 0; i_count < VLVC_MAX_USERS; i_count++ )
        welcome_data.p_meeting_auth_users[i_count] = p_vlvc->p_meeting_auth_users[i_count];

    if( ServerNetWrite( p_vlvc, i_user_index, &c_command,
                        sizeof( c_command ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );

    if( ServerNetWrite( p_vlvc, i_user_index, &welcome_data,
                        sizeof( welcome_data ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: SendCodecsData
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             int    i_user_index : the client id to send the codecs data to
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called to send the codecs data when the server forces it
 *****************************************************************************/
static int SendCodecsData( vlvc_t *p_vlvc, int i_user_index )
{
    vlvc_codecs_info_t       codecs_data;
    char                     c_command = VLVC_NET_COMMAND_CODECS;
    vlvc_param_t             *p_parameters = p_vlvc->p_parameters;

    memcpy(codecs_data.psz_video_codec, p_parameters->psz_video_codec,
           sizeof( codecs_data.psz_video_codec ) );
    memcpy(codecs_data.psz_audio_codec, p_parameters->psz_audio_codec,
           sizeof( codecs_data.psz_audio_codec ) );
    codecs_data.i_video_bitrate = p_parameters->i_video_bitrate;
    codecs_data.i_audio_bitrate = p_parameters->i_audio_bitrate;

    if( ServerNetWrite( p_vlvc, i_user_index, &c_command,
                        sizeof( c_command ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );

    if( ServerNetWrite( p_vlvc, i_user_index, &codecs_data,
                        sizeof( codecs_data ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: GetClientIpAddress
 *
 * Parameters:
 *             int  i_fd           : file descriptor associated with the IP
 *                                   address to retreive
 *             char *psz_client_ip : output buffer to store the IP address
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Retreives the IP address associated with the given file
 *             descriptor.
 *****************************************************************************/
static int GetClientIpAddress(int i_fd, char psz_client_ip[VLVC_MAX_IP_ADDRESS_LENGTH])
{
    int i_client_port;

    if( net_GetPeerAddress( i_fd, psz_client_ip, &i_client_port ) != VLC_SUCCESS )
    {
        net_Close(i_fd);
        return( VLC_EGENERIC );
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ServerInitUserInfo
 *
 * Parameters:
 *             vlvc_t      *p_vlvc         : a pointer to the vlvc object
 *             vlvc_user_t *p_current_user : output user data to fill.
 *             int         i_fd            : file descriptor to use to read
 *                                           user's information
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called when a user sends a hello message to retreive
 *             his information.
 *****************************************************************************/
static int ServerInitUserInfo( vlvc_t *p_vlvc, vlvc_user_t *p_current_user, int i_fd )
{
    vlvc_client_user_info_t     user_info;

    p_current_user->i_fd = i_fd;
    p_current_user->b_free = VLC_FALSE;
    p_vlvc->i_nb_users++;
    if( ServerNetRead( p_vlvc, i_fd, (void *) &user_info,
                       sizeof( user_info ) ) == VLVC_READ_ERROR )
        return( VLVC_CLIENT_QUIT );

    memset( p_current_user->psz_name, STRING_TERMINATOR,
            sizeof( p_current_user->psz_name ) );
    memset( p_current_user->psz_description, STRING_TERMINATOR,
            sizeof( p_current_user->psz_description ) );
    memset( p_current_user->psz_password, STRING_TERMINATOR,
            sizeof( p_current_user->psz_password ) );
    memset( p_current_user->psz_ip_address, STRING_TERMINATOR,
            sizeof( p_current_user->psz_ip_address ) );
    memset( p_current_user->psz_version, STRING_TERMINATOR,
            sizeof( p_current_user->psz_version ) );

    strncpy(  p_current_user->psz_name, user_info.psz_name,
              sizeof( user_info.psz_name ) );
    strncpy(  p_current_user->psz_description,
              user_info.psz_description, sizeof( user_info.psz_description ) );
    strncpy(  p_current_user->psz_password,
              user_info.psz_password, sizeof( user_info.psz_password ) );
    strncpy(  p_current_user->psz_version,
              user_info.psz_version, sizeof( user_info.psz_version ) );
    p_current_user->i_first_client_port = user_info.i_first_stream_port;
    p_current_user->b_multicast = user_info.b_multicast;

    if( GetClientIpAddress( i_fd, p_current_user->psz_ip_address ) == VLC_EGENERIC )
       return( VLVC_CLIENT_QUIT );

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: InitSelfInfo
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int
 *
 * Description:
 *             Inits our user structure using the parameters provided by the
 *             user.
 *****************************************************************************/
static int InitSelfInfo( vlvc_t *p_vlvc )
{
    vlvc_user_t *p_current_user = &p_vlvc->p_users[p_vlvc->i_self_index];

    p_current_user->i_fd = VLVC_INVALID_FD;
    p_current_user->b_free = VLC_FALSE;

    memset( p_current_user->psz_name, STRING_TERMINATOR,
            sizeof( p_current_user->psz_name ) );
    memset( p_current_user->psz_description, STRING_TERMINATOR,
            sizeof( p_current_user->psz_description ) );
    memset( p_current_user->psz_ip_address, STRING_TERMINATOR,
            sizeof( p_current_user->psz_ip_address ) );

    strncpy( p_current_user->psz_name, p_vlvc->p_parameters->psz_username,
             sizeof( p_current_user->psz_name ) );
    strncpy( p_current_user->psz_description, p_vlvc->p_parameters->psz_desc,
             sizeof( p_current_user->psz_description ) );

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: SelfJoin
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *             char   *psz_vlm_output_command : VLM command used for the
 *                                              configuration
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Used to add oneself to the conference.
 *****************************************************************************/
static int SelfJoin( vlvc_t *p_vlvc, char psz_vlm_output_command[VLVC_MAX_VLM_COMMAND_LEN] )
{
    vlvc_param_t        *p_parameters = p_vlvc->p_parameters;
    int                 i_opt_idx;

    InitSelfInfo( p_vlvc );

    if( CreateVlmUserControl( p_vlvc, p_vlvc->i_self_index ) != VLC_SUCCESS )
        return( VLVC_VLM_CREATE );

    if( SetupInputVlmUserControl( p_vlvc, p_vlvc->i_self_index,
                                  p_parameters->psz_input_mrl ) != VLC_SUCCESS )
        return( VLVC_VLM_SETUP );

    for( i_opt_idx = 0; p_parameters->psz_input_options[i_opt_idx] != NULL; i_opt_idx++ )
    {
        if( SetupOptionVlmUserControl( p_vlvc, p_vlvc->i_self_index,
                                       p_parameters->psz_input_options[i_opt_idx] ) != VLC_SUCCESS )
            return( VLVC_VLM_OPTION );
    }

    if( SetupOutputVlmUserControl( p_vlvc, p_vlvc->i_self_index,
                                   psz_vlm_output_command ) != VLC_SUCCESS )
        return( VLVC_VLM_SETUP );

    if( PlayVlmUserControl( p_vlvc, p_vlvc->i_self_index ) != VLC_SUCCESS )
        return( VLVC_VLM_SETUP );

    return( VLC_SUCCESS );
}


/*****************************************************************************
 * Name: ServerSelfJoin
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called on the server to add himself to the conference
 *****************************************************************************/
static int ServerSelfJoin( vlvc_t *p_vlvc )
{
    vlvc_param_t        *p_parameters = p_vlvc->p_parameters;
    char                psz_vlm_command[VLVC_MAX_VLM_COMMAND_LEN];
    int                 i_ret;

    if( p_parameters->b_standalone == VLC_TRUE )
        return( VLC_SUCCESS );

    p_vlvc->i_self_index = VLVC_SERVER_DEFAULT_INDEX;
    snprintf( psz_vlm_command, sizeof( psz_vlm_command ),
              VLVC_SPRINTF_SELF_SERVER_OUTPUT_MRL,
              p_parameters->psz_video_codec, p_parameters->i_video_bitrate,
              p_parameters->psz_video_scale,
              p_parameters->psz_audio_codec, p_parameters->i_audio_bitrate );

    i_ret = SelfJoin( p_vlvc, psz_vlm_command );

    p_vlvc->i_nb_users++;
    p_vlvc->DisableHandBtn( NULL );
    if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING ||
        p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
    {
        p_vlvc->EnableAuthBtn( NULL );
        p_vlvc->EnableUnauthBtn( NULL );
    }
    return i_ret;
}

/*****************************************************************************
 * Name: ServerUserJoin
 *
 * Parameters:
 *             vlvc_t *p_vlvc     : a pointer to the vlvc object
 *             int    i_client_id : new client id
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called on the server when a new user joins to setup
 *             vlm controls.
 *****************************************************************************/
static int ServerUserJoin( vlvc_t *p_vlvc, int i_user_index)
{
    vlvc_param_t        *p_parameters = p_vlvc->p_parameters;

    printf("\n================ REINIT =================\n"); //AFF
    if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CHAT_ROOM ||
        p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING ||
        p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
    {
        char            psz_vlm_command[VLVC_MAX_VLM_COMMAND_LEN];

        if( CreateVlmUserControl( p_vlvc, i_user_index ) != VLC_SUCCESS )
            return( VLVC_VLM_CREATE );

        snprintf( psz_vlm_command, sizeof( psz_vlm_command ),
                  VLVC_SPRINTF_SERVER_INPUT_MRL,
                  p_parameters->psz_bind_address,
                  p_vlvc->p_parameters->i_first_client_port + i_user_index );
        if( SetupInputVlmUserControl( p_vlvc, i_user_index,
                                      psz_vlm_command ) != VLC_SUCCESS )
            return( VLVC_VLM_SETUP );
        if( p_parameters->b_standalone == VLC_FALSE ||
            ( p_parameters->b_standalone == VLC_TRUE && p_vlvc->i_nb_users >= 2 ) )
        {
            SetupClientsOutputsChatRoom( p_vlvc );

            if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CHAT_ROOM )
            {
                if( PlayVlmUserControl( p_vlvc, i_user_index ) != VLC_SUCCESS )
                    return( VLVC_VLM_SETUP );
            }
        }
    }
    else if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CONFERENCE )
         SetupClientsOutputsConference( p_vlvc );
    else
        return( VLC_EGENERIC );
    printf("\n=============== END REINIT ==============\n"); //AFF

    GuiNbClientChange( p_vlvc, i_user_index );
    NotifyUserJoin( p_vlvc, i_user_index );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: SendNotWelcome
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             int    i_user_index : the client id to send not welcome to
 *             char   *psz_error_description : not welcome message
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called to send a not welcome message to the user.
 *****************************************************************************/
static int SendNotWelcome( vlvc_t *p_vlvc, int i_user_index, char *psz_error_description )
{
    char                     c_command = VLVC_NET_COMMAND_NOT_WELCOME;
    vlvc_not_welcome_data_t  not_welcome_data;

    ServerNetWrite( p_vlvc, i_user_index, &c_command, sizeof( c_command ) );
    snprintf( not_welcome_data.psz_error_description,
              VLVC_BUFFER_SIZE( not_welcome_data.psz_error_description ),
              psz_error_description );
    ServerNetWrite( p_vlvc, i_user_index, &not_welcome_data,
                    sizeof( not_welcome_data ) );
    return ( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: HandleHello
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             int    i_fd         : client file descriptor
 *             int    *i_client_id : client id to be filled
 *
 * Returns:
 *             int: VLVC return code, used to know if we need to quit
 *
 * Description:
 *             Function called when a user sends a hello message.
 *****************************************************************************/
static int HandleHello( vlvc_t *p_vlvc, int i_fd, int *i_user_index )
{
    *i_user_index = GetUserId( p_vlvc );

    if( *i_user_index < 0 )
    {
        SendNotWelcome( p_vlvc, *i_user_index, NULL );
        return( VLVC_CLIENT_QUIT );
    }
    else
    {
        msg_Dbg( p_vlvc, "[VLVC]new user : %d\n", *i_user_index );
        if( ServerInitUserInfo( p_vlvc, &p_vlvc->p_users[*i_user_index],
                                i_fd ) != VLC_SUCCESS )
            return( VLVC_CLIENT_QUIT );
        if ( HandleVersion( p_vlvc, &p_vlvc->p_users[*i_user_index], *i_user_index ) != VLC_SUCCESS )
        {
            p_vlvc->p_users[*i_user_index].i_fd = 0;
            p_vlvc->p_users[*i_user_index].b_free = VLC_TRUE;
            p_vlvc->i_nb_users--;
            return ( VLVC_CLIENT_QUIT );
        }
        if ( HandlePassword( p_vlvc, &p_vlvc->p_users[*i_user_index], *i_user_index ) != VLC_SUCCESS )
        {
            p_vlvc->p_users[*i_user_index].i_fd = 0;
            p_vlvc->p_users[*i_user_index].b_free = VLC_TRUE;
            p_vlvc->i_nb_users--;
            return ( VLVC_CLIENT_QUIT );
        }
        if( SendWelcome( p_vlvc, *i_user_index ) != VLC_SUCCESS )
            return( VLVC_CLIENT_QUIT );

        if( p_vlvc->p_users[*i_user_index].b_multicast == VLC_TRUE &&
            p_vlvc->p_parameters->b_multicast == VLC_TRUE )
        {
            SendMulticastInfo( p_vlvc, *i_user_index );
            msleep( 10000 ); //a voir
            ServerTryMulticast( p_vlvc, *i_user_index );
            ReceiveMulticastReply( p_vlvc, *i_user_index );
        }
        else
            p_vlvc->p_users[*i_user_index].b_multicast = VLC_FALSE;

        if( p_vlvc->p_parameters->b_force_codecs == VLC_TRUE )
            if( SendCodecsData( p_vlvc, *i_user_index ) != VLC_SUCCESS )
                return( VLVC_CLIENT_QUIT );
        ServerUserJoin( p_vlvc, *i_user_index );
        return( VLC_SUCCESS );
    }
}

/*****************************************************************************
 * Name: NetSendMessage
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             vlvc_message_header_t *message_header : message header to send
 *             char   *psz_message : message to send
 *             int    i_fd         : recipient fd
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called by both the server and the client to send
 *             a message throught the specified fd
 *****************************************************************************/
static int NetSendMessage( vlvc_t *p_vlvc, vlvc_message_header_t *message_header,
                    const char *psz_message, int i_fd )
{
    char c_command = VLVC_NET_COMMAND_MESSAGE;

    if( NetWrite( p_vlvc, i_fd, &c_command,
                  sizeof( c_command ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );
    if( NetWrite( p_vlvc, i_fd, message_header,
                  sizeof( *message_header ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );
    if( NetWrite( p_vlvc, i_fd, psz_message,
                  message_header->i_size * sizeof( *psz_message ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ClientNetSendMessage
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             vlvc_message_header_t *message_header : message header to send
 *             char   *psz_message : message to send
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called by the client to send a message to the server
 *****************************************************************************/
static int ClientNetSendMessage( vlvc_t *p_vlvc, vlvc_message_header_t *message_header,
                          const char *psz_message )
{
    return( NetSendMessage( p_vlvc, message_header, psz_message,
                            p_vlvc->i_client_socket ) );
}

/*****************************************************************************
 * Name: ServerNetSendMessage
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             vlvc_message_header_t *message_header : message header to send
 *             char   *psz_message : message to send
 *             int    i_user_index : recipient user index
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called by the server to sends a message to the specified
 *             user
 *****************************************************************************/
static int ServerNetSendMessage( vlvc_t *p_vlvc, vlvc_message_header_t *message_header,
                          const char *psz_message, int i_user_index )
{
    return( NetSendMessage( p_vlvc, message_header, psz_message,
                            p_vlvc->p_users[i_user_index].i_fd ) );
}

/*****************************************************************************
 * Name: ServerNetSendMessageAll
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             vlvc_message_header_t *message_header : message header to send
 *             char   *psz_message : message to send
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called by the server to send a message to all the users
 *             and to display the message in the GUI.
 *****************************************************************************/
static int ServerNetSendMessageAll( vlvc_t *p_vlvc, vlvc_message_header_t *message_header,
                             const char *psz_message )
{
    int         i_current_user;

    for( i_current_user = 0; i_current_user < p_vlvc->i_nb_users; i_current_user++ )
        if( i_current_user != p_vlvc->i_self_index )
            ServerNetSendMessage( p_vlvc, message_header, psz_message,
                                  i_current_user );
    GuiDisplayMessage( p_vlvc, message_header, psz_message );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ReceiveMessage
 *
 * Parameters:
 *             vlvc_t *p_vlvc       : a pointer to the vlvc object
 *             vlvc_message_header_t *message_header : message header to be
 *                                                     filled
 *             char   **psz_message : message content to be filled
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             When a message is received, fills its header and content
 *****************************************************************************/
static int ReceiveMessage( vlvc_t *p_vlvc, vlvc_message_header_t *message_header,
                    char **psz_message, int i_user_index )
{
    ServerNetRead( p_vlvc, p_vlvc->p_users[i_user_index].i_fd,
                   message_header, sizeof( *message_header ) );

    if( message_header->i_size > 0 )
        *psz_message = malloc( message_header->i_size * sizeof( **psz_message ) );
    else
        return( VLC_EGENERIC );

    if( *psz_message == NULL)
        return( VLC_ENOMEM );

    if( ServerNetRead( p_vlvc, p_vlvc->p_users[i_user_index].i_fd, *psz_message,
                       message_header->i_size * sizeof( **psz_message ) ) == VLVC_READ_ERROR )
    {
        free( *psz_message );
        return( VLC_EGENERIC );
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: HandleMessage
 *
 * Parameters:
 *             vlvc_t *p_vlvc     : a pointer to the vlvc object
 *             int    i_client_id : client id who sent the message
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called when a user sends a message
 *****************************************************************************/
static void HandleMessage( vlvc_t *p_vlvc, int i_user_index )
{
    vlvc_message_header_t    message_header;
    char                     *psz_message = NULL;

    if( ReceiveMessage( p_vlvc, &message_header,
                         &psz_message, i_user_index ) != VLC_SUCCESS )
        return;
    if( message_header.c_user_dest_id == VLVC_NET_MESSAGE_ALL )
        ServerNetSendMessageAll( p_vlvc, &message_header, psz_message  );
    else if( message_header.c_user_dest_id > 0 &&
              message_header.c_user_dest_id < p_vlvc->i_nb_users )
        ServerNetSendMessage( p_vlvc, &message_header, psz_message, i_user_index );
    else
        printf( "[VLVC]Invalid client id : %d to send message\n", message_header.c_user_dest_id );

    free( psz_message );
}

/*****************************************************************************
 * Name: AddToTemporaryUsers
 *
 * Parameters:
 *             vlvc_t *p_vlvc : pointer to the vlvc object
 *             int    i_fd    : fd to add in the temporary users list
 *
 * Returns:
 *             void
 *
 * Description:
 *             Adds the given fd to the temporary users list
 *****************************************************************************/
static void AddToTemporaryUsers(vlvc_t *p_vlvc, int i_fd)
{
    int i_pos;

    for( i_pos = 0; i_pos < VLVC_MAX_USERS; i_pos++)
        if( p_vlvc->p_pending_fd[i_pos] == VLVC_INVALID_FD )
        {
            p_vlvc->p_pending_fd[i_pos] = i_fd;
            p_vlvc->i_pending_users++;
            break;
        }
}

/*****************************************************************************
 * Name: ReadOnListeningSocket
 *
 * Parameters:
 *             vlvc_t * p_vlvc : pointer to the vlvc object
 *             fd_set *fds_r : pointer to the select()set fds
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Handles specifically on the listening socket. When select() sets
 *             this fd, that means that someone is connecting to the server.
 *             So, this function is dedicated to accept new clients.
 *****************************************************************************/
static int ReadOnListeningSocket(vlvc_t *p_vlvc, fd_set *fds_r)
{
    int         *p_listen_fd = p_vlvc->p_listen_fd;

    if( p_vlvc->i_pending_users < VLVC_MAX_USERS &&
        FD_ISSET( *p_listen_fd, fds_r ) )
    {
        int    i_new_client_fd;
        if( ( i_new_client_fd = net_Accept( p_vlvc, p_listen_fd, -1 ) ) != -1 )
            AddToTemporaryUsers( p_vlvc, i_new_client_fd );
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: HandleLeavingUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc           : pointer to the vlvc object
 *             int    i_leaving_user_id : the leaving user's ID
 *
 * Returns:
 *             int: VLVC return code
 *
 * Description:
 *             Does what has to be done when a client has left.
 *****************************************************************************/
static void HandleLeavingUser(vlvc_t *p_vlvc, int i_leaving_user_id)
{
    SendLeaveUser( p_vlvc, i_leaving_user_id );
    ServerUserLeave( p_vlvc, i_leaving_user_id );
}

/*****************************************************************************
 * Name: HandleVersion
 *
 * Parameters:
 *             vlvc_t *p_vlvc        : pointer to the vlvc object
 *             int    i_user_id      : the concerned user's ID
 *             char   c_message_type : the message type received from the client
 *
 * Returns:
 *
 *
 * Description:
 *             Checks if the client's version matches the server's version
 *****************************************************************************/
static int HandleVersion(vlvc_t *p_vlvc, vlvc_user_t *p_current_user, int i_user_index )
{
    char psz_error_description[VLVC_MAX_ERROR_MESSAGE_LENGTH];

    if( strcmp( p_vlvc->p_parameters->psz_version,
                 p_current_user->psz_version ) )
    {
        snprintf( psz_error_description, VLVC_BUFFER_SIZE( psz_error_description ),
                  VLVC_CORE_MODULE_ERROR_INCORRECT_VERSION,
                  p_vlvc->p_parameters->psz_version, p_current_user->psz_version );
        SendNotWelcome( p_vlvc, i_user_index, psz_error_description );
        return ( VLC_EGENERIC );
    }
    return ( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: HandlePassword
 *
 * Parameters:
 *             vlvc_t *p_vlvc        : pointer to the vlvc object
 *             int    i_user_id      : the concerned user's ID
 *             char   c_message_type : the message type received from the client
 *
 * Returns:
 *
 *
 * Description:
 *             Checks if the client's password matches with the password
 *             set on server.
 *****************************************************************************/
static int HandlePassword(vlvc_t *p_vlvc, vlvc_user_t *p_current_user, int i_user_index )
{
    if( !strcmp( p_vlvc->p_parameters->psz_password, "" ) )
        return ( VLC_SUCCESS );
    if( strcmp( p_vlvc->p_parameters->psz_password,
                p_current_user->psz_password ) )
    {
        SendNotWelcome( p_vlvc, i_user_index, VLVC_CORE_MODULE_ERROR_INCORRECT_PASSWORD );
        return ( VLC_EGENERIC );
    }
    return ( VLC_SUCCESS );
}


/*****************************************************************************
 * Name: HandleRaiseHand
 *
 * Parameters:
 *             vlvc_t *p_vlvc     : a pointer to the vlvc object
 *             int    i_client_id : client id who sent the message
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called when a user sends a "Raise hand" request
 *****************************************************************************/
static void HandleRaiseHand( vlvc_t *p_vlvc, int i_user_index )
{
    char                        psz_raise_hand_message[VLVC_SPRINTF_RAISE_HAND_MESSAGE_MAX_SIZE];
    vlvc_message_header_t       message_header;
    int                         i_message_size = 0;


    i_message_size = snprintf( psz_raise_hand_message,
                               VLVC_BUFFER_SIZE( psz_raise_hand_message ),
                               VLVC_SPRINTF_RAISE_HAND_MESSAGE,
                               p_vlvc->p_users[i_user_index].psz_name );
    message_header.c_user_src_id = i_user_index;
    message_header.c_type = VLVC_NET_MESSAGE_TYPE_INFO;
    message_header.i_size = i_message_size;
    GuiDisplayMessage( p_vlvc, &message_header, psz_raise_hand_message);
}

/*****************************************************************************
 * Name: HandleMessageFromClient
 *
 * Parameters:
 *             vlvc_t * p_vlvc       : pointer to the vlvc object
 *             int    i_user_id      : the concerned user's ID
 *             char   c_message_type : the message type received from the client
 *
 * Returns:
 *             int: VLVC return code
 *
 * Description:
 *             Does what has to be done with the received command.
 *****************************************************************************/
static int HandleMessageFromClient(vlvc_t *p_vlvc, int i_user_id,
                                   char c_message_type)
{
    switch( c_message_type )
    {
    case VLVC_NET_COMMAND_PING:
        msg_Dbg( p_vlvc, "[VLVC]PING\n");
        break;
    case VLVC_NET_COMMAND_QUIT:
        msg_Dbg( p_vlvc, "[VLVC]QUIT\n");
        HandleLeavingUser( p_vlvc, i_user_id );
        break;
    case VLVC_NET_COMMAND_MESSAGE:
        msg_Dbg( p_vlvc, "[VLVC]MESSAGE\n");
        HandleMessage( p_vlvc, i_user_id );
        break;
    case VLVC_NET_COMMAND_HAND:
        msg_Dbg ( p_vlvc, "[VLVC]HAND\n" );
        HandleRaiseHand( p_vlvc, i_user_id );
        break;
    default:
        msg_Dbg( p_vlvc, "vlvc unkown message type %d", c_message_type );
		break;
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ReadOnSetClientFD
 *
 * Parameters:
 *             vlvc_t * p_vlvc : pointer to the vlvc object
 *             fd_set *fds_r   : pointer to the select()set fds
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Loops on all the clients, and handle the incoming message if
 *             his fd has been set by select().
 *****************************************************************************/
static int ReadOnSetClientFD(vlvc_t *p_vlvc, fd_set *fds_r)
{
    int         i_cur_user;
    char        c_message_type = VLVC_NET_COMMAND_QUIT;

    for( i_cur_user = 0; i_cur_user < VLVC_MAX_USERS; i_cur_user++ )
    {
        int    i_fd = p_vlvc->p_users[i_cur_user].i_fd;
        if( i_cur_user != p_vlvc->i_self_index &&
            i_fd != VLVC_INVALID_FD &&
            p_vlvc->p_users[i_cur_user].b_free == VLC_FALSE )
            {
                if( FD_ISSET( i_fd, fds_r ) )
                {
                    int i_ret_read = ServerNetRead( p_vlvc, i_fd,
                                                    (void *) &c_message_type,
                                                    sizeof( c_message_type ) );
                    if( i_ret_read != VLVC_READ_ERROR )
                    {
                        if( i_ret_read == 0 )
                            HandleLeavingUser( p_vlvc, i_cur_user );
                        else
                            HandleMessageFromClient( p_vlvc, i_cur_user,
                                                    c_message_type );
                    }
                    else
                        HandleLeavingUser( p_vlvc, i_cur_user );
                }
            }
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ReadOnTemporaryFD
 *
 * Parameters:
 *             vlc_object_t *p_object : pointer to the common object structure
 *             fd_set       *fds_r    : pointer to the select()set fds
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Handles users that have not been accepted yet.
 *****************************************************************************/
static int ReadOnTemporaryFD(vlvc_t *p_vlvc, fd_set *fds_r)
{
    int    i_cur_user;

    for( i_cur_user = 0; i_cur_user < VLVC_MAX_USERS; i_cur_user++ )
    {
        int    i_fd = p_vlvc->p_pending_fd[i_cur_user];

        if( i_fd != VLVC_INVALID_FD )
        {
            if( FD_ISSET( i_fd, fds_r ) )
            {
                char    c_message_type = VLVC_NET_COMMAND_QUIT;

                if( ServerNetRead( p_vlvc, i_fd, (void *) &c_message_type,
                                   sizeof( c_message_type ) ) != VLVC_READ_ERROR )
                {
                    if( c_message_type == VLVC_NET_COMMAND_HELLO )
                    {
                        int i_user_index = VLVC_INVALID_CLIENT_ID;
                        if( HandleHello( p_vlvc, i_fd,
                                         &i_user_index ) == VLVC_CLIENT_QUIT )
                            net_Close( i_fd );
                    }
                    else
                        net_Close( i_fd );
                }
                p_vlvc->p_pending_fd[i_cur_user] = VLVC_INVALID_FD;
                p_vlvc->i_pending_users--;
            }
        }
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ReadFromClient
 *
 * Parameters:
 *             vlc_object_t *p_object : pointer to the common object structure
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Reads client messages depending on select() set fd.
 *****************************************************************************/
static int ReadFromClient( vlc_object_t* p_object, fd_set *fds_r )
{
    vlvc_t      *p_vlvc = (vlvc_t *)p_object;

    ReadOnListeningSocket( p_vlvc, fds_r );
    ReadOnSetClientFD( p_vlvc, fds_r );
    ReadOnTemporaryFD( p_vlvc, fds_r );

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: DoServer
 *
 * Parameters:
 *             vlc_object_t *p_object : pointer to the common object structure
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Used by the server to wait for connections.
 *             calls select in a loop and read on fd if necessary.
 *****************************************************************************/
static int DoServer( vlc_object_t* p_object )
{
    vlvc_t              *p_vlvc = (vlvc_t *)p_object;
    int                 i_ret;

    do
    {
        struct timeval  timeout;
        fd_set          fds_r/*, fds_e*/;
        int             i_cur_user;
        int             i_max_fd = *p_vlvc->p_listen_fd;

        FD_ZERO( &fds_r );
        FD_SET( *p_vlvc->p_listen_fd, &fds_r );
        for( i_cur_user = 0; i_cur_user < VLVC_MAX_USERS; i_cur_user++ )
        {
            if( p_vlvc->p_users[i_cur_user].b_free == VLC_FALSE &&
                p_vlvc->p_users[i_cur_user].i_fd != VLVC_INVALID_FD )
                if( i_cur_user != p_vlvc->i_self_index )
                {
                    FD_SET( p_vlvc->p_users[i_cur_user].i_fd, &fds_r );
                    if( p_vlvc->p_users[i_cur_user].i_fd > i_max_fd )
                        i_max_fd = p_vlvc->p_users[i_cur_user].i_fd;
                }
            if( p_vlvc->p_pending_fd[i_cur_user] != VLVC_INVALID_FD )
            {
                FD_SET( p_vlvc->p_pending_fd[i_cur_user], &fds_r );
                if( p_vlvc->p_pending_fd[i_cur_user] > i_max_fd )
                    i_max_fd = p_vlvc->p_pending_fd[i_cur_user];
            }
        }
        timeout.tv_sec = 0;
        timeout.tv_usec = 500000;
        i_ret = select( i_max_fd + 1, &fds_r, NULL, NULL, &timeout );
        if( i_ret > 0 )
            ReadFromClient( p_object, &fds_r );
    }while( i_ret >= 0 && p_vlvc->b_die == VLC_FALSE );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ServerMode
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called when VLVC is started in server mode
 *****************************************************************************/
static int ServerMode( vlvc_t *p_vlvc )
{
    vlvc_param_t    *p_parameters = p_vlvc->p_parameters;
    int             i_cur_user;

    msg_Dbg( p_vlvc, "[VLVC] Launching standalone server on : %s:%d\n",
           p_vlvc->p_parameters->psz_bind_address,
           p_vlvc->p_parameters->i_bind_port);

    for( i_cur_user = 0; i_cur_user < VLVC_MAX_USERS; i_cur_user++ )
    {
        memset( &p_vlvc->p_users[i_cur_user], 0,
                sizeof( p_vlvc->p_users[i_cur_user] ) );
        p_vlvc->p_users[i_cur_user].b_free = VLC_TRUE;
        p_vlvc->p_users[i_cur_user].i_fd = VLVC_INVALID_FD;
    }

    p_vlvc->p_listen_fd = net_ListenTCP( p_vlvc, p_parameters->psz_bind_address,
                          p_parameters->i_bind_port );
    if( p_vlvc->p_listen_fd > 0 )
    {
        GenerateMulticastInfo( p_vlvc );
        if( ServerSelfJoin( p_vlvc ) != VLC_SUCCESS )
            return( VLC_EGENERIC );
        GuiNbClientChange( p_vlvc, p_vlvc->i_self_index );

        vlc_mutex_init( p_vlvc, &p_vlvc->accept_lock );
        if( vlc_thread_create( p_vlvc, VLVC_SERVER_THREAD, DoServer,
                               VLC_THREAD_PRIORITY_LOW, VLC_FALSE ) )
        {
            p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_SERVER_THREAD_INDEX;
            p_vlvc->b_started = VLC_FALSE;
            return( VLC_EGENERIC );
        }
        p_vlvc->b_started = VLC_TRUE;
        return( VLC_SUCCESS );
    }
    p_vlvc->b_started = VLC_FALSE;
    p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_BIND_INDEX;
    return( VLC_EGENERIC );
}

/*****************************************************************************
 * Name: ClientInitUserInfo
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Initializes the new user data sent by the server
 *****************************************************************************/
static int     ClientInitUserInfo( vlvc_t *p_vlvc, int *i_new_user_id )
{
    vlvc_server_user_info_t     add_user_data;
    vlvc_user_t                 *p_user;

    if( ClientNetRead( p_vlvc, &add_user_data,
                        sizeof( add_user_data ) ) == VLVC_READ_ERROR )
    {
        p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_GET_USER_INFO_INDEX;
        return( VLC_EGENERIC );
    }

    if( add_user_data.c_user_id < 0 ||
         add_user_data.c_user_id >= VLVC_MAX_USERS )
    {
        p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_USER_ID_OUT_OF_RANGE_INDEX;
        return( VLC_EGENERIC );
    }

    *i_new_user_id = (int)add_user_data.c_user_id;

    p_user = &p_vlvc->p_users[*i_new_user_id];
    memset(p_user, STRING_TERMINATOR, sizeof( *p_user ) );
    strncpy( p_user->psz_name, add_user_data.psz_name,
             sizeof( p_user->psz_name ) );
    strncpy( p_user->psz_description, add_user_data.psz_description,
             sizeof( p_user->psz_description ) );
    p_user->b_free = VLC_FALSE;
    p_vlvc->i_nb_users++;

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ClientSelfJoin
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called on the client when the server informs this client himself
 *             has joined. In this case, the client has to play its webcam on
 *             the display and send it to the server.
 *****************************************************************************/
static int ClientSelfJoin( vlvc_t *p_vlvc )
{
    vlvc_param_t        *p_parameters = p_vlvc->p_parameters;
    char                psz_vlm_command[VLVC_MAX_VLM_COMMAND_LEN];

    snprintf( psz_vlm_command, sizeof( psz_vlm_command ),
              VLVC_SPRINTF_SELF_CLIENT_OUTPUT_MRL,
              p_parameters->psz_video_codec, p_parameters->i_video_bitrate,
              p_parameters->psz_video_scale,
              p_parameters->psz_audio_codec, p_parameters->i_audio_bitrate,
              p_parameters->psz_network_protocol,
              p_parameters->psz_server_address,
              p_vlvc->i_server_base_stream_port + p_vlvc->i_self_index );

    return( SelfJoin( p_vlvc, psz_vlm_command ) );
}

/*****************************************************************************
 * Name: ClientPlayUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             int    i_user_index : user to play
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called on the client when a new user joins to setup
 *             VLM controls.
 *****************************************************************************/
static int ClientPlayUser( vlvc_t *p_vlvc, int i_user_index )
{
    char            psz_vlm_command[VLVC_MAX_VLM_COMMAND_LEN];
    int             i_dest_user;

    if( CreateVlmUserControl( p_vlvc, i_user_index ) != VLC_SUCCESS )
        return( VLVC_VLM_CREATE );

    if( p_vlvc->b_server_multicast == VLC_TRUE)
    {
        snprintf( psz_vlm_command, sizeof( psz_vlm_command ),
                  VLVC_SPRINTF_CLIENT_MULTICAST_INPUT_MRL,
                  p_vlvc->psz_multicast_group,
                  VLVC_FIRST_CLIENT_PORT_DEFAULT + i_user_index );
    }
    else
    {
        snprintf( psz_vlm_command, sizeof( psz_vlm_command ),
                  VLVC_SPRINTF_CLIENT_INPUT_MRL,
                  p_vlvc->p_parameters->i_first_client_port + i_user_index );
    }

    if( SetupInputVlmUserControl( p_vlvc, i_user_index,
                                  psz_vlm_command ) != VLC_SUCCESS )
        return( VLVC_VLM_SETUP );
    if( i_user_index == p_vlvc->i_server_index )
    {
        if( PlayVlmUserControl( p_vlvc, i_user_index ) != VLC_SUCCESS )
            return( VLVC_VLM_SETUP );
    }
    else
    {
        if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CHAT_ROOM )
        {
            if( PlayVlmUserControl( p_vlvc, i_user_index ) != VLC_SUCCESS )
                return( VLVC_VLM_SETUP );
        }
        else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
        {
            if( p_vlvc->i_amphi_auth_user == i_user_index )
            {
                if( PlayVlmUserControl( p_vlvc, i_user_index ) != VLC_SUCCESS )
                    return( VLVC_VLM_SETUP );
            }
        }
        else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
        {
            for( i_dest_user = 0; i_dest_user < VLVC_MAX_USERS; i_dest_user++ )
                if ( p_vlvc->p_meeting_auth_users[i_dest_user] == i_user_index )
                {
                    if( PlayVlmUserControl( p_vlvc, i_user_index ) != VLC_SUCCESS )
                        return( VLVC_VLM_SETUP );
                }
        }
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ClientUserJoin
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called on the client when a new user joins.
 *             First, we get the new user information, then, depending on the
 *             conference mode, we have to add ourself to the conference, or
 *             read other clients' stream, then inform the GUI of this new user.
 *****************************************************************************/
static int ClientUserJoin( vlvc_t *p_vlvc)
{
    vlvc_param_t        *p_parameters = p_vlvc->p_parameters;
    int                 i_user_index = VLVC_INVALID_CLIENT_ID;
    int                 i_ret = VLC_EGENERIC;

    if( ClientInitUserInfo( p_vlvc, &i_user_index ) != VLC_SUCCESS )
        return( VLC_EGENERIC );

    if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CHAT_ROOM )
    {
        if( i_user_index == p_vlvc->i_self_index )
            i_ret = ClientSelfJoin( p_vlvc );
        else
            i_ret = ClientPlayUser( p_vlvc, i_user_index );
    }
    else if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CONFERENCE )
    {
        if( i_user_index == p_vlvc->i_server_index )
            i_ret = ClientPlayUser( p_vlvc, i_user_index );
    }
    else if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
    {
        if( i_user_index == p_vlvc->i_self_index )
            i_ret = ClientSelfJoin( p_vlvc );
        else
            i_ret = ClientPlayUser( p_vlvc, i_user_index );
    }
    else if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
    {
        if( i_user_index == p_vlvc->i_self_index )
            i_ret = ClientSelfJoin( p_vlvc );
        else
            i_ret = ClientPlayUser( p_vlvc, i_user_index );
    }
    GuiNbClientChange( p_vlvc, i_user_index );
    return( i_ret );
}

/*****************************************************************************
 * Name: ClientUserLeave
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called on the client when a user leaves.
 *             We first have to read the leaving client ID, then stop its VLM
 *             controls and clean its data. Then inform the GUI.
 *****************************************************************************/
static int ClientUserLeave( vlvc_t *p_vlvc )
{
    vlvc_param_t        *p_parameters = p_vlvc->p_parameters;
    char                c_user_index = VLVC_INVALID_CLIENT_ID;
    int                 i_count;

    ClientNetRead( p_vlvc, &c_user_index, sizeof( c_user_index ) );
    if( c_user_index > 0 && c_user_index < VLVC_MAX_USERS )
    {
        if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CHAT_ROOM ||
            p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING ||
            p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
        {
            vlvc_user_t *current_user = &p_vlvc->p_users[(int) c_user_index];
            //memset( current_user->psz_name, STRING_TERMINATOR, VLVC_MAX_USER_NAME_LENGTH );
            memset( current_user->psz_description, STRING_TERMINATOR, VLVC_MAX_USER_DESC_LENGTH );
            memset( current_user->psz_ip_address, STRING_TERMINATOR, VLVC_MAX_IP_ADDRESS_LENGTH );

            StopVlmUserControl( p_vlvc, (int) c_user_index );
            DeleteVlmUserControl( p_vlvc, (int) c_user_index );

            current_user->i_fd = VLVC_INVALID_FD;
            current_user->b_free = VLC_TRUE;
            current_user->b_multicast = VLC_FALSE;
            p_vlvc->i_nb_users--;
            GuiNbClientChange( p_vlvc, (int) c_user_index );
            if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
            {
                if( p_vlvc->i_amphi_auth_user == (int) c_user_index )
                    p_vlvc->i_amphi_auth_user = VLVC_AUTH_FREE;
            }
            else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
            {
                for( i_count = 0; i_count < VLVC_MAX_USERS; i_count++ )
                    if( p_vlvc->p_meeting_auth_users[i_count] == (int) c_user_index )
                        p_vlvc->p_meeting_auth_users[i_count] = VLVC_AUTH_FREE;
            }
        }
        else if( p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CONFERENCE )
        {
            GuiNbClientChange( p_vlvc, (int) c_user_index );
        }
        else
            return( VLC_EGENERIC );
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ClientUserAuthorize
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called on the client when a user is authorized.
 *             We first have to read the leaving client ID, then stop its VLM
 *             controls and clean its data. Then inform the GUI.
 *****************************************************************************/
static int ClientUserAuthorize( vlvc_t *p_vlvc )
{
    int                i_user_index = VLVC_INVALID_CLIENT_ID;
    int                i_dest_user;
    int                i_test_auth;

    ClientNetRead( p_vlvc, &i_user_index, sizeof( i_user_index ) );
    i_test_auth = 0;
    if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
    {
        vlvc_StopUser( p_vlvc, p_vlvc->i_amphi_auth_user );
        p_vlvc->i_amphi_auth_user = i_user_index;
        i_test_auth = 1;
    }
    else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
    {
        for( i_dest_user = 0; i_dest_user < VLVC_MAX_USERS &&
                 !i_test_auth; i_dest_user++ )
            if ( p_vlvc->p_meeting_auth_users[i_dest_user] == i_user_index )
                i_test_auth++;
        if ( !i_test_auth )
        {
            for( i_dest_user = 0; i_dest_user < VLVC_MAX_USERS &&
                     !i_test_auth; i_dest_user++ )
                if ( p_vlvc->p_meeting_auth_users[i_dest_user] == VLVC_AUTH_FREE )
                {
                    i_test_auth++;
                    p_vlvc->p_meeting_auth_users[i_dest_user] = i_user_index;
                }
        }
    }
    if( i_user_index > 0 && i_user_index < VLVC_MAX_USERS && i_test_auth )
        PlayVlmUserControl( p_vlvc, i_user_index );
    GuiAuthUser( p_vlvc, i_user_index );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ClientUserUnauthorize
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called on the client when a user has been unauthorized.
 *             We first have to read the leaving client ID, then stop its VLM
 *             controls and clean its data. Then inform the GUI.
 *****************************************************************************/
static int ClientUserUnauthorize( vlvc_t *p_vlvc )
{
    int                i_user_index = VLVC_INVALID_CLIENT_ID;
    int                i_count;

    ClientNetRead( p_vlvc, &i_user_index, sizeof( i_user_index ) );
    if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
        p_vlvc->i_amphi_auth_user = VLVC_AUTH_FREE;
    else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
    {
        for( i_count = 0; i_count < VLVC_MAX_USERS; i_count++ )
            if( p_vlvc->p_meeting_auth_users[i_count] == i_user_index )
                p_vlvc->p_meeting_auth_users[i_count] = VLVC_AUTH_FREE;
    }
    if( i_user_index > 0 && i_user_index < VLVC_MAX_USERS &&
        i_user_index != p_vlvc->i_self_index )
        vlvc_StopUser( p_vlvc, i_user_index );
    GuiUnauthUser( p_vlvc, i_user_index );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: SendHello
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function called on the client to send hello to the server
 *****************************************************************************/
static int SendHello( vlvc_t *p_vlvc )
{
    vlvc_client_user_info_t       user_info;
    char                          c_command = VLVC_NET_COMMAND_HELLO;

    memcpy( user_info.psz_name, p_vlvc->p_parameters->psz_username,
             sizeof( user_info.psz_name ) );
    memcpy( user_info.psz_description, p_vlvc->p_parameters->psz_desc,
             sizeof( user_info.psz_description ) );
    memcpy( user_info.psz_password, p_vlvc->p_parameters->psz_password,
             sizeof( user_info.psz_password ) );
    memcpy( user_info.psz_version, p_vlvc->p_parameters->psz_version,
             sizeof( user_info.psz_version ) );
    user_info.i_first_stream_port = p_vlvc->p_parameters->i_first_client_port;
    user_info.b_multicast = p_vlvc->p_parameters->b_multicast;

    if( ClientNetWrite( p_vlvc, &c_command,
                        sizeof( c_command ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );

    if( ClientNetWrite( p_vlvc, &user_info,
                        sizeof( user_info ) ) == sizeof( user_info ) )
        return( VLC_SUCCESS );
    return( VLC_EGENERIC );
}

/*****************************************************************************
 * Name: ClientReceiveWelcomeData
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the client to receive the conference information.
 *****************************************************************************/
static int    ClientReceiveWelcomeData( vlvc_t *p_vlvc )
{
    vlvc_welcome_data_t         welcome_data;
    int                         i_count;
    char                        psz_conference_infos_message[VLVC_SPRINTF_CONFERENCE_INFOS_MAX_SIZE];
    vlvc_message_header_t       message_header;
    int                         i_message_size = 0;
    char                        psz_conference_string[VLVC_CONFERENCE_STRING_MAX_LEN];

    if( ClientNetRead( p_vlvc, (void *) &welcome_data,
                       sizeof( welcome_data ) ) == VLVC_READ_ERROR )
    {
        p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_GET_CONFERENCE_DATA_INDEX;
        return( VLC_EGENERIC );
    }
    p_vlvc->i_self_index = welcome_data.c_user_id;
    p_vlvc->p_parameters->i_conference_mode = (int)welcome_data.c_conference_mode;
    p_vlvc->i_server_index = welcome_data.c_server_id;
    p_vlvc->i_server_base_stream_port = welcome_data.i_first_stream_port;
    p_vlvc->b_server_multicast = welcome_data.b_multicast;
    p_vlvc->i_amphi_auth_user = welcome_data.i_amphi_auth_user;
    for( i_count = 0; i_count < VLVC_MAX_USERS; i_count++ )
        p_vlvc->p_meeting_auth_users[i_count] = welcome_data.p_meeting_auth_users[i_count];

    switch( p_vlvc->p_parameters->i_conference_mode )
    {
    case VLVC_CONFERENCE_MODE_CHAT_ROOM:
        snprintf( psz_conference_string, VLVC_BUFFER_SIZE( psz_conference_string ),
                  VLVC_CONFERENCE_STRING_CHAT_ROOM );
        break;
    case VLVC_CONFERENCE_MODE_CONFERENCE:
        snprintf( psz_conference_string, VLVC_BUFFER_SIZE( psz_conference_string ),
                  VLVC_CONFERENCE_STRING_CONFERENCE );
        break;
    case VLVC_CONFERENCE_MODE_MEETING:
        snprintf( psz_conference_string, VLVC_BUFFER_SIZE( psz_conference_string ),
                  VLVC_CONFERENCE_STRING_MEETING );
        break;
    case VLVC_CONFERENCE_MODE_AMPHITHEATRE:
        snprintf( psz_conference_string, VLVC_BUFFER_SIZE( psz_conference_string ),
                  VLVC_CONFERENCE_STRING_AMPHITHEATRE );
        break;
    default:
        msg_Dbg( p_vlvc, "Undefined conference mode" );
    }

    i_message_size = snprintf( psz_conference_infos_message,
                               VLVC_BUFFER_SIZE( psz_conference_infos_message ),
                               VLVC_SPRINTF_CONFERENCE_INFOS,
                               welcome_data.psz_server_name,
                               psz_conference_string );

    message_header.c_user_src_id = p_vlvc->i_server_index;
    message_header.c_user_dest_id = p_vlvc->i_self_index;
    message_header.c_type = VLVC_NET_MESSAGE_TYPE_INFO;
    message_header.i_size = i_message_size;

    GuiDisplayMessage( p_vlvc, &message_header, psz_conference_infos_message );

    if ( p_vlvc->p_parameters->i_conference_mode != VLVC_CONFERENCE_MODE_MEETING &&
         p_vlvc->p_parameters->i_conference_mode != VLVC_CONFERENCE_MODE_AMPHITHEATRE )
    {
        p_vlvc->DisableHandBtn( NULL );
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ClientReceiveNotWelcomeData
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the client to receive the not welcome data.
 *****************************************************************************/
static int ClientReceiveNotWelcomeData( vlvc_t *p_vlvc )
{
    vlvc_not_welcome_data_t not_welcome_data;

    if( ClientNetRead( p_vlvc, (void *) &not_welcome_data,
                       sizeof( not_welcome_data ) ) == VLVC_READ_ERROR )
    {
        p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_GET_NOT_WELCOME_COMMAND_INDEX;
        return( VLC_EGENERIC );
    }
    if( not_welcome_data.psz_error_description )
    {
        GuiMessageBox( p_vlvc, not_welcome_data.psz_error_description,
                       VLVC_CRITICAL_LEVEL_MAX );
        return( VLC_EGENERIC );
    }
    GuiMessageBox( p_vlvc, VLVC_CORE_MODULE_ERROR_REFUSED,
                   VLVC_CRITICAL_LEVEL_MAX );
    return( VLC_EGENERIC );
}

/*****************************************************************************
 * Name: ReceiveWelcome
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Used by the client to receive the welcome message from the
 *             server.
 *****************************************************************************/
static int ReceiveWelcome( vlvc_t *p_vlvc )
{
    int         i_ret = VLC_EGENERIC;
    char        message_type;

    if( ClientNetRead( p_vlvc, &message_type,
                        sizeof( message_type )) == VLVC_READ_ERROR )
    {
        p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_GET_WELCOME_COMMAND_INDEX;
        return( VLC_EGENERIC );
    }
    switch( message_type )
    {
    case VLVC_NET_COMMAND_WELCOME:
        i_ret = ClientReceiveWelcomeData( p_vlvc );
        break;
    case VLVC_NET_COMMAND_NOT_WELCOME:
        ClientReceiveNotWelcomeData( p_vlvc );
        break;
    default:
        msg_Dbg( p_vlvc, "Unknown message."); // a changer
    }
    return( i_ret );
}

/*****************************************************************************
 * Name: ClientReceiveMessage
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function used by the client to receive a message.
 *****************************************************************************/
static int ClientReceiveMessage( vlvc_t *p_vlvc )
{
    vlvc_message_header_t       message_header;
    char                        *psz_message;

    if( ClientNetRead( p_vlvc, &message_header,
                       sizeof( message_header ) ) == VLVC_READ_ERROR )
        return( VLC_EGENERIC );

    if( message_header.i_size > 0 &&
        message_header.c_user_src_id < VLVC_MAX_USERS )
        psz_message = malloc( message_header.i_size );
    else
        return( VLC_EGENERIC );

    if( psz_message == NULL)
        return( VLC_ENOMEM );

    if( ClientNetRead( p_vlvc, psz_message,
                       message_header.i_size ) != VLVC_READ_ERROR )
        if( p_vlvc->p_users[(int)message_header.c_user_src_id].b_free == VLC_FALSE )
            GuiDisplayMessage( p_vlvc, &message_header, psz_message );
    free( psz_message );
    return( VLC_SUCCESS );
}


/*****************************************************************************
 * Name: ClientReceiveCodecsData
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Function used by the client to receive the codecs to use when
 *             forced by the server.
 *****************************************************************************/
static int ClientReceiveCodecsData( vlvc_t *p_vlvc )
{
    vlvc_codecs_info_t          codecs_data;
    vlvc_param_t                *p_parameters = p_vlvc->p_parameters;
    char                        psz_force_codec_message[VLVC_SPRINTF_FORCE_CODEC_MESSAGE_MAX_SIZE];
    vlvc_message_header_t       message_header;
    int                         i_message_size = 0;

    if( ClientNetRead( p_vlvc, &codecs_data,
                       sizeof( codecs_data ) ) == VLVC_READ_ERROR )
        return( VLC_EGENERIC );

    memcpy( p_parameters->psz_video_codec, codecs_data.psz_video_codec,
            sizeof( p_parameters->psz_video_codec ) );

    memcpy( p_parameters->psz_audio_codec, codecs_data.psz_audio_codec,
            sizeof( p_parameters->psz_audio_codec ) );

    p_parameters->i_video_bitrate = codecs_data.i_video_bitrate;
    p_parameters->i_audio_bitrate = codecs_data.i_audio_bitrate;

    i_message_size = snprintf( psz_force_codec_message,
                               sizeof( psz_force_codec_message ),
                               VLVC_SPRINTF_FORCE_CODEC_MESSAGE,
                               p_parameters->psz_video_codec,
                               p_parameters->i_video_bitrate,
                               p_parameters->psz_audio_codec,
                               p_parameters->i_audio_bitrate );

    message_header.c_user_src_id =  p_vlvc->i_server_index;
    message_header.c_user_dest_id = p_vlvc->i_self_index;
    message_header.c_type = VLVC_NET_MESSAGE_TYPE_INFO;
    message_header.i_size = i_message_size;

    GuiDisplayMessage( p_vlvc, &message_header, psz_force_codec_message );

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: DoClient
 *
 * Parameters:
 *             vlc_object_t * p_object : pointer to the common object structure
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Used by the client to process server messages in a loop.
 *             This function is launched as a thread.
 *****************************************************************************/
static int DoClient( vlc_object_t* p_object )
{
    vlvc_t          *p_vlvc = (vlvc_t *)p_object;
    char            message_type;

    if( SendHello( p_vlvc ) != VLC_SUCCESS )
    {
       net_Close( p_vlvc->i_client_socket );
       return( VLC_SUCCESS );
    }
    if( ReceiveWelcome( p_vlvc ) != VLC_SUCCESS )
    {
        net_Close( p_vlvc->i_client_socket );
        return( VLC_SUCCESS );
    }
    if( p_vlvc->b_server_multicast == VLC_TRUE )
    {
        ReceiveMulticastInfo( p_vlvc );
        ClientTryMulticast( p_vlvc );
    }
    while( ClientNetRead( p_vlvc, (void *) &message_type,
                           sizeof( message_type ) ) != VLVC_READ_ERROR )
    {
        vlc_mutex_lock( &p_vlvc->command_lock );
        switch(message_type)
        {
        case VLVC_NET_COMMAND_ADD_USER:
            msg_Dbg( p_vlvc, "[VLVC]DoClient it's a new user\n");
            ClientUserJoin( p_vlvc );
            break;
        case VLVC_NET_COMMAND_LEAVE_USER:
            msg_Dbg( p_vlvc, "[VLVC]DoClient a user leaves\n");
            ClientUserLeave( p_vlvc );
            break;
        case VLVC_NET_COMMAND_AUTHORIZE:
            msg_Dbg( p_vlvc, "[VLVC]DoClient a user is authorized\n");
            ClientUserAuthorize( p_vlvc );
            break;
        case VLVC_NET_COMMAND_UNAUTHORIZE:
            msg_Dbg( p_vlvc, "[VLVC]DoClient a user is unauthorized\n");
            ClientUserUnauthorize( p_vlvc );
            break;
        case VLVC_NET_COMMAND_CODECS:
            msg_Dbg( p_vlvc, "[VLVC]Codec Forced\n");
            ClientReceiveCodecsData( p_vlvc );
            break;
        case VLVC_NET_COMMAND_MESSAGE:
            msg_Dbg( p_vlvc, "[VLVC]MESSAGE\n");
            ClientReceiveMessage( p_vlvc );
            break;
        }
        vlc_mutex_unlock( &p_vlvc->command_lock );
    }
    GuiMessageBox( p_vlvc, VLVC_CORE_MODULE_ERROR_CONNECTION_LOST,
                   VLVC_CRITICAL_LEVEL_MAX );
    net_Close( p_vlvc->i_client_socket );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ClientMode
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLC return code
 *
 * Description:
 *             Used on the client to connect to the server. Once connected
 *             successfully, the server messages processing loop thread is
 *             created.
 *****************************************************************************/
static int ClientMode( vlvc_t *p_vlvc )
{
    msg_Dbg( p_vlvc, "[VLVC]Launching client connecting to server: %s:%d\n",
           p_vlvc->p_parameters->psz_server_address,
           p_vlvc->p_parameters->i_server_port);

    p_vlvc->i_client_socket = net_ConnectTCP( p_vlvc,
                          p_vlvc->p_parameters->psz_server_address,
                          p_vlvc->p_parameters->i_server_port );
    if( p_vlvc->i_client_socket < 0 )
    {
        p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_CONNECT_INDEX;
        return( VLC_EGENERIC );
    }


    if( vlc_thread_create( p_vlvc, VLVC_CLIENT_THREAD, DoClient,
                           VLC_THREAD_PRIORITY_LOW, VLC_FALSE ) )
    {
        p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_CLIENT_THREAD_INDEX;
        return( VLC_EGENERIC );
    }
    p_vlvc->b_started = VLC_TRUE;
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: HandleVlmUserControl
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *             char   *psz_vlm_command_type : Used as a basis to create the
 *                                            VLM command.
 *             char   *psz_mrl  : additional command parameters to complete
 *                                the VLM command.
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Generic function that executes the provided VLM command.
 *
 * Comments:
 * Called with a sprintf template, this function just have to add the user id
 * and some additional information if the psz_mrl is not NULL.
 * If psz_mrl is not NULL, then the psz_vlm_command_type has a ending %s that
 * parmits us to add psz_mrl.
 * Refer to the callers and the macros they use for a better understanding.
 * We have a timing problem under Windows, which obliged us to add a msleep.
 * This is a bad solution, but the best we found.
 *****************************************************************************/
static int HandleVlmUserControl( vlvc_t *p_vlvc, int i_user_id,
                                 char *psz_vlm_command_type, char *psz_mrl )
{
    vlm_message_t       *message;
    char                psz_vlm_command[VLVC_MAX_VLM_COMMAND_LEN];

    if( p_vlvc == NULL || p_vlvc->p_vlm == NULL || i_user_id < 0 ||
         i_user_id >= VLVC_MAX_USERS )
        return( VLVC_EARGS );

    if( psz_mrl != NULL )
        snprintf( psz_vlm_command, VLVC_MAX_VLM_COMMAND_LEN, psz_vlm_command_type,
                  (int) i_user_id, psz_mrl );
    else
        snprintf( psz_vlm_command, VLVC_MAX_VLM_COMMAND_LEN, psz_vlm_command_type,
                  (int) i_user_id);
    vlm_ExecuteCommand( p_vlvc->p_vlm, psz_vlm_command, &message );
    msg_Dbg( p_vlvc, "vlm command : %s", psz_vlm_command );
    printf( "vlm command : %s\n", psz_vlm_command );
    vlm_MessageDelete( message );

#if defined (WIN32)
    msleep( 200000 );
#endif

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: SetupVideoWindowTitle
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Used to set the window title for the new video window.
 *****************************************************************************/
static int SetupVideoWindowTitle( vlvc_t *p_vlvc, int i_user_id )
{
    char psz_title_option[VLVC_MAX_TITLE_OPTION_LEN];
    char psz_user_name[VLVC_MAX_USER_NAME_LENGTH * 2];

    CopyAndAddBackslashes( psz_user_name, p_vlvc->p_users[i_user_id].psz_name,
                           sizeof( psz_user_name ) );

    snprintf( psz_title_option, sizeof( psz_title_option ),
              VLVC_SPRINTF_VIDEO_TITLE_OPTION, psz_user_name );

    return( SetupOptionVlmUserControl( p_vlvc, i_user_id, psz_title_option ) );
}

/*****************************************************************************
 * Name: SetCacheValue
 *
 * Parameters:
 *             vlvc_t *p_vlvc         : a pointer to the vlvc object
 *             int    i_user_id       : user ID
 *             char   *psz_cache_name : VLM option used to change the value
 *             int    i_cache_value   : the value to set
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Setup the given caching value.
 *****************************************************************************/
static int SetCacheValue( vlvc_t *p_vlvc, int i_user_id,
                          char *psz_cache_name, int i_cache_value )
{
    if( i_cache_value != VLVC_IGNORE_VALUE )
    {
        char psz_caching_buf[VLVC_SPRINTF_CACHING_MAX_SIZE];

        snprintf( psz_caching_buf, sizeof( psz_caching_buf ),
                  psz_cache_name, i_cache_value );

        return( SetupOptionVlmUserControl( p_vlvc, i_user_id, psz_caching_buf ) );
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: SetupCachingValues
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Setup every caching values.
 *
 * Comment:
 * The caching values are stored in the parameters structure.
 * if a value is VLVC_IGNORE_VALUE , the user selected "Default", and we do not
 * change any value.
 *****************************************************************************/
static int SetupCachingValues( vlvc_t *p_vlvc, int i_user_id )
{
    int i_ret;
    vlvc_caching_values_t *caching_values = &p_vlvc->p_parameters->caching_values;

    i_ret = SetCacheValue( p_vlvc, i_user_id,
                                    VLVC_SPRINTF_CACHING_VIDEO_IN_OPTION,
                                    caching_values->i_video_input );
    if( i_ret != VLC_SUCCESS )
        return i_ret;

    i_ret = SetCacheValue( p_vlvc, i_user_id,
                            VLVC_SPRINTF_CACHING_UDP_OUT_OPTION,
                            caching_values->i_udp_output );
    if( i_ret != VLC_SUCCESS )
        return i_ret;

    i_ret = SetCacheValue( p_vlvc, i_user_id,
                            VLVC_SPRINTF_CACHING_UDP_IN_OPTION,
                            caching_values->i_udp_input );
    if( i_ret != VLC_SUCCESS )
        return i_ret;

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: SetupOverlay
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Enables/disables the overlay for the specified client's video
 *             window.
 *****************************************************************************/
static int SetupOverlay( vlvc_t *p_vlvc, int i_user_id )
{
    if( p_vlvc->p_parameters->b_fix_overlay == VLC_TRUE )
        return( SetupOptionVlmUserControl( p_vlvc, i_user_id,
                                           VLVC_OPTION_NO_OVERLAY ) );
    return( VLC_SUCCESS );
}
/*****************************************************************************
 * Name: CreateVlmUserControl
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Wrapper for HandleVlmUserControl only designed to make the code
 *             easier to be read.
 *             Creates the VLM control of the specified user.
 *****************************************************************************/
static int CreateVlmUserControl( vlvc_t *p_vlvc, int i_user_id )
{
    return( HandleVlmUserControl( p_vlvc, i_user_id, VLVC_CREATE_VLM_CONTROL_COMMAND, NULL ) );
}

/*****************************************************************************
 * Name: PlayVlmUserControl
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Wrapper for HandleVlmUserControl only designed to make the code
 *             easier to be read.
 *             Plays the VLM control of the specified user.
 *****************************************************************************/
static int PlayVlmUserControl( vlvc_t *p_vlvc, int i_user_id )
{
    SetupVideoWindowTitle( p_vlvc, i_user_id );
    SetupCachingValues( p_vlvc, i_user_id );
    SetupOverlay( p_vlvc, i_user_id );

    return( HandleVlmUserControl( p_vlvc, i_user_id, VLVC_PLAY_VLM_CONTROL_COMMAND, NULL ) );
}

/*****************************************************************************
 * Name: StopVlmUserControl
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Wrapper for HandleVlmUserControl only designed to make the code
 *             easier to be read.
 *             Stops the VLM control of the specified user.
 *****************************************************************************/
static int StopVlmUserControl( vlvc_t *p_vlvc, int i_user_id )
{
    return( HandleVlmUserControl( p_vlvc, i_user_id, VLVC_STOP_VLM_CONTROL_COMMAND, NULL ) );
}

/*****************************************************************************
 * Name: DeleteVlmUserControl
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Wrapper for HandleVlmUserControl only designed to make the code
 *             easier to be read.
 *             Deletes the VLM control of the specified user.
 *****************************************************************************/
static int DeleteVlmUserControl( vlvc_t *p_vlvc, int i_user_id )
{
    return( HandleVlmUserControl( p_vlvc, i_user_id, VLVC_DEL_VLM_CONTROL_COMMAND, NULL ) );
}

/*****************************************************************************
 * Name: SetupInputVlmUserControl
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *             char   *psz_mrl  : main MRL data
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Wrapper for HandleVlmUserControl only designed to make the code
 *             easier to be read.
 *             Sets up the input VLM control of the specified user.
 *****************************************************************************/
static int SetupInputVlmUserControl( vlvc_t *p_vlvc, int i_user_id, char *psz_mrl )
{
    return( HandleVlmUserControl( p_vlvc, i_user_id, VLVC_SETUP_INPUT_VLM_CONTROL_COMMAND, psz_mrl ) );
}

/*****************************************************************************
 * Name: SetupOptionVlmUserControl
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *             char   *psz_mrl  : main MRL data
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Wrapper for HandleVlmUserControl only designed to make the code
 *             easier to be read.
 *             Sets up the options of the specified user VLM control.
 *****************************************************************************/
static int SetupOptionVlmUserControl( vlvc_t *p_vlvc, int i_user_id, char *psz_mrl  )
{
    return( HandleVlmUserControl( p_vlvc, i_user_id, VLVC_SETUP_OPTION_VLM_CONTROL_COMMAND, psz_mrl ) );
}

/*****************************************************************************
 * Name: SetupOutputVlmUserControl
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : user ID
 *             char   *psz_mrl  : main MRL data
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Wrapper for HandleVlmUserControl only designed to make the code
 *             easier to be read.
 *             Sets up the output VLM control of the specified user.
 *****************************************************************************/
static int SetupOutputVlmUserControl( vlvc_t *p_vlvc, int i_user_id, char *psz_mrl  )
{
    return( HandleVlmUserControl( p_vlvc, i_user_id, VLVC_SETUP_OUTPUT_VLM_CONTROL_COMMAND, psz_mrl ) );
}

/*****************************************************************************
 * Name: GenerateMulticastInfo
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Generate a multicast IP address and a port for multicast use.
 *****************************************************************************/
static int GenerateMulticastInfo( vlvc_t *p_vlvc )
{
    int         i_loop;

    memset( p_vlvc->psz_multicast_group, 0, VLVC_MAX_IP_ADDRESS_LENGTH );
    srand( time( NULL ) );
    sprintf( p_vlvc->psz_multicast_group, "225" );
    for( i_loop = 0; i_loop < 3; i_loop++ )
        sprintf( p_vlvc->psz_multicast_group, "%s.%d", p_vlvc->psz_multicast_group,
                 (int)( ( (float)rand( ) / (float)RAND_MAX * 256 ) ) );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: SendMulticastInfo
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             int    i_user_index : the user id to send multicast info to
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Send the multicast IP address to the specified user.
 *****************************************************************************/
static int SendMulticastInfo( vlvc_t *p_vlvc, int i_user_index )
{
    vlvc_multicast_info_t multicast_info;
    char                  c_command = VLVC_NET_COMMAND_MULTICAST_INFO;

    memcpy( multicast_info.psz_multicast_group, p_vlvc->psz_multicast_group,
            sizeof( multicast_info.psz_multicast_group ) );

    if( ServerNetWrite( p_vlvc, i_user_index, &c_command,
                        sizeof( c_command ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );

    if( ServerNetWrite( p_vlvc, i_user_index, &multicast_info,
                        sizeof( multicast_info ) ) == sizeof( multicast_info ) )
        return( VLC_SUCCESS );
    return( VLC_EGENERIC );
}

/*****************************************************************************
 * Name: ReceiveMulticastInfo
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Used by the client to receive the multicast info message from
 *             the server.
 *****************************************************************************/
static int ReceiveMulticastInfo( vlvc_t *p_vlvc )
{
    vlvc_multicast_info_t multicast_info;
    char                  message_type;

    if( ClientNetRead( p_vlvc, &message_type,
                       sizeof( message_type ) ) == VLVC_READ_ERROR )
    {
        p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_GET_MULTICAST_COMMAND_INDEX;
        return( VLC_EGENERIC );
    }
    if( message_type == VLVC_NET_COMMAND_MULTICAST_INFO )
    {
        if( ClientNetRead( p_vlvc, (void *) &multicast_info,
                           sizeof( multicast_info ) ) == VLVC_READ_ERROR )
        {
            p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_GET_MULTICAST_DATA_INDEX;
            return( VLC_EGENERIC );
        }
        memcpy( p_vlvc->psz_multicast_group, multicast_info.psz_multicast_group,
                sizeof( p_vlvc->psz_multicast_group ) );
        return( VLC_SUCCESS );
    }
    else
        msg_Dbg( p_vlvc, "multicast : Unknown message" );
    return( VLC_EGENERIC );
}

/*****************************************************************************
 * Name: ServerTryMulticast
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             int    i_user_index : the user id to make the test with
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             The server will try to send a multicast message on the
 *             generated multicast IP.
 *****************************************************************************/
static int ServerTryMulticast( vlvc_t *p_vlvc, int i_user_index )
{
    char               psz_message[] = VLVC_MULTICAST_CHECK_MESSAGE;
    struct sockaddr_in socket_struct;
    unsigned char      c_ttl = 1;
    int                i_socket;
    int                i_length;

    if( ( i_socket = socket( PF_INET, SOCK_DGRAM, 0 ) ) < 0 )
        return( VLC_EGENERIC );
    memset( &socket_struct, 0, sizeof( socket_struct ) );
    socket_struct.sin_family = AF_INET;
    socket_struct.sin_port = htons( VLVC_FIRST_CLIENT_PORT_DEFAULT + i_user_index );
    socket_struct.sin_addr.s_addr = inet_addr( p_vlvc->psz_multicast_group );

    i_length = sizeof( socket_struct );
    if( ( setsockopt( i_socket, IPPROTO_IP, IP_MULTICAST_TTL, &c_ttl,
                      sizeof( c_ttl ) ) ) == -1 )
        return( VLC_EGENERIC );

    if( ( sendto( i_socket, psz_message, sizeof( psz_message ), 0,
                  (struct sockaddr *)&socket_struct, i_length ) ) < 0 )
        return( VLC_EGENERIC );
    net_Close( i_socket );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ClientTryMulticast
 *
 * Parameters:
 *             vlvc_t *p_vlvc: a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             The client will try to receive message from the multicast IP.
 *****************************************************************************/
static int ClientTryMulticast( vlvc_t *p_vlvc )
{
    int                i_socket;
    struct sockaddr_in socket_struct;
    int                i_length;
    struct ip_mreq     imr_struct;
    char               buf[1024];
    char               c_command;
    struct timeval     timeout;
    fd_set             fds_r;

    if( ( i_socket = socket( PF_INET, SOCK_DGRAM, 0 ) ) < 0 )
        return( VLC_EGENERIC );
    memset( &socket_struct, 0, sizeof( socket_struct ) );
    socket_struct.sin_family = AF_INET;
    socket_struct.sin_port = htons( VLVC_FIRST_CLIENT_PORT_DEFAULT
                                    + p_vlvc->i_self_index );
    socket_struct.sin_addr.s_addr = htonl( INADDR_ANY );

    i_length = sizeof( socket_struct );
    if( bind(i_socket, (struct sockaddr *)&socket_struct, sizeof(socket_struct)) < 0 )
        return( VLC_EGENERIC );

    imr_struct.imr_multiaddr.s_addr = inet_addr( p_vlvc->psz_multicast_group );
    imr_struct.imr_interface.s_addr = htonl( INADDR_ANY );
    if( setsockopt( i_socket, IPPROTO_IP, IP_ADD_MEMBERSHIP, (void *) &imr_struct,
                    sizeof( struct ip_mreq ) ) < 0 )
        return( VLC_EGENERIC );

    FD_ZERO( &fds_r );
    FD_SET( i_socket, &fds_r );
    timeout.tv_sec = 0;
    timeout.tv_usec = 100000;
    if( select( i_socket + 1, &fds_r, NULL, NULL, &timeout ) > 0 )
    {
        if( ( recvfrom( i_socket, buf, sizeof(buf), 0, (struct sockaddr *)&socket_struct,
                        (socklen_t *)&i_length ) ) <= 0 )
            return( VLC_EGENERIC );
        if ( strcmp( buf, VLVC_MULTICAST_CHECK_MESSAGE ) == 0 )
            c_command = VLVC_NET_COMMAND_MULTICAST_OK;
        else
            c_command = VLVC_NET_COMMAND_MULTICAST_KO;
    }
    else
    {
        c_command = VLVC_NET_COMMAND_MULTICAST_KO;
    }
    net_Close( i_socket );

    if( ClientNetWrite( p_vlvc, &c_command, sizeof( c_command ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );

    if( c_command == VLVC_NET_COMMAND_MULTICAST_OK )
        p_vlvc->b_server_multicast = VLC_TRUE;
    else
        p_vlvc->b_server_multicast = VLC_FALSE;

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: ReceiveMulticastReply
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             int    i_user_index : the user id to read from
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             The server receives the reply of the client from the multicast
 *             test.
 *****************************************************************************/
static int ReceiveMulticastReply( vlvc_t *p_vlvc, int i_user_index )
{
    char                c_message_type;

     if( ServerNetRead( p_vlvc, p_vlvc->p_users[i_user_index].i_fd,
                       (void *) &c_message_type,
                       sizeof( c_message_type ) ) == VLVC_READ_ERROR )
        return( VLC_EGENERIC );
    if( c_message_type == VLVC_NET_COMMAND_MULTICAST_OK )
    {
        p_vlvc->p_users[i_user_index].b_multicast = VLC_TRUE;
    }
    else
    {
        p_vlvc->p_users[i_user_index].b_multicast = VLC_FALSE;
        if( c_message_type != VLVC_NET_COMMAND_MULTICAST_KO )
            return( VLC_EGENERIC );
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: __vlvc_CheckUp
 *
 * Parameters:
 *             vlc_object_t * p_this : a pointer to the common object structure
 *
 * Returns:
 *             int : VLC_TRUE or VLC_FALSE
 *
 * Description:
 *             Checks if the vlvc module is ok for the startup
 *             (cf. ../open.cpp).
 *****************************************************************************/
int __vlvc_CheckUp( vlc_object_t *p_this )
{
    vlc_value_t lockval;
    vlvc_t      *p_vlvc = NULL;
    int         i_ret = VLC_TRUE;

    var_Create( p_this->p_libvlc, VLVC_MUTEX, VLC_VAR_MUTEX );
    var_Get( p_this->p_libvlc, VLVC_MUTEX, &lockval );
    vlc_mutex_lock( lockval.p_address );
    p_vlvc = vlc_object_find( p_this, VLC_OBJECT_VLVC, FIND_ANYWHERE );
    vlc_mutex_unlock( lockval.p_address );

    if( p_vlvc == NULL )
        return( i_ret );
    p_vlvc->i_refcount -= 1;
    if( p_vlvc->b_initialized != VLC_TRUE )
        return( i_ret );
    vlc_mutex_lock( &p_vlvc->common_lock );
    if( p_vlvc->b_started == VLC_TRUE )
    {
        i_ret = VLC_FALSE;
        GuiMessageBox( p_vlvc, VLVC_CORE_MODULE_ERROR_ALREADY_LAUNCHED,
                       VLVC_CRITICAL_LEVEL_QUIT_LIMT );
    }
    vlc_mutex_unlock( &p_vlvc->common_lock );
    return( i_ret );
}

/*****************************************************************************
 * Name: vlvc_Init
 *
 * Parameters:
 *             vlvc_t       *p_vlvc       : a pointer to the vlvc object
 *             vlvc_param_t *p_parameters : pointer to the VLVC parameters
 *                                          structure
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Initializes the vlvc object.
 *
 * Comments:
 * The paraneters structure is allocated and freed in the access module.
 * So the given pointer does not have to be copied or anything else.
 *****************************************************************************/
int vlvc_Init( vlvc_t *p_vlvc, vlvc_param_t *p_new_parameters )
{
    if( p_vlvc == NULL || p_new_parameters == NULL )
        return( VLVC_EARGS );
    vlc_mutex_lock( &p_vlvc->common_lock );
    p_vlvc->p_parameters = p_new_parameters;
    p_vlvc->b_initialized = VLC_TRUE;
    vlc_mutex_unlock( &p_vlvc->common_lock );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_Start
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Starts the core module in the right mode.
 *****************************************************************************/
int vlvc_Start( vlvc_t *p_vlvc )
{
    int                 i_ret = VLC_EGENERIC;
    vlvc_param_t        *p_parameters = p_vlvc->p_parameters;

    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    if( p_vlvc->b_initialized != VLC_TRUE )
        return( VLVC_NOT_INITIALIZED );

    vlc_mutex_lock( &p_vlvc->common_lock );
    if( p_vlvc->b_started == VLC_TRUE )
    {
        // Rajouter les msg_Err en plus
        if( p_vlvc->b_GUIPresent == VLC_TRUE &&
             p_parameters->b_standalone == VLC_TRUE )
            p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_STANDALONE_GUI_INDEX;
        else
            p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_ALREADY_LAUNCHED_INDEX;
        return( VLC_EGENERIC );
    }
    if( p_parameters->b_standalone == VLC_TRUE &&
        p_parameters->i_operating_mode == VLVC_CLIENT_MODE )
    {
        p_vlvc->i_last_error = VLVC_CORE_MODULE_ERROR_INVALID_STANDALONE_INDEX;
        return( VLC_EGENERIC );
    }
    if( p_parameters->i_operating_mode == VLVC_SERVER_MODE )
        i_ret = ServerMode( p_vlvc );
    else if( p_parameters->i_operating_mode == VLVC_CLIENT_MODE )
        i_ret = ClientMode( p_vlvc );
    vlc_mutex_unlock( &p_vlvc->common_lock );
    return( i_ret );
}

/*****************************************************************************
 * Name: vlvc_Stop
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Stops the core module.
 *****************************************************************************/
int vlvc_Stop( vlvc_t *p_vlvc )
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    if( p_vlvc->p_parameters == NULL )
        return( VLC_EGENERIC );
    GuiStop( p_vlvc );
    playlist_t * p_playlist =
        (playlist_t *)vlc_object_find( p_vlvc, VLC_OBJECT_PLAYLIST,
                                       FIND_ANYWHERE );
    if( p_playlist == NULL )
    {
        return( VLC_EGENERIC );
    }
    playlist_Stop( p_playlist );
    vlc_object_release( p_playlist );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_NetSendMessage
 *
 * Parameters:
 *             vlvc_t *p_vlvc      : a pointer to the vlvc object
 *             char   *psz_message : message to send
 *             int    i_user_dest  : user to send the message to
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Callback used by the GUI when the user wants to send a message.
 *****************************************************************************/
int vlvc_NetSendMessage( vlvc_t *p_vlvc, const char *psz_message, int i_user_dest )
{
    vlvc_message_header_t message_header;

    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    if( p_vlvc->p_parameters == NULL )
        return( VLC_EGENERIC );

    message_header.c_user_src_id = p_vlvc->i_self_index;
    message_header.c_user_dest_id = (char) i_user_dest;
    message_header.c_type = VLVC_NET_MESSAGE_TYPE_NORMAL;
    message_header.i_size = strlen( psz_message ) + sizeof( *psz_message );

    if( p_vlvc->p_parameters->i_operating_mode == VLVC_SERVER_MODE )
        ServerNetSendMessageAll( p_vlvc, &message_header, psz_message );
    else
        ClientNetSendMessage( p_vlvc, &message_header, psz_message );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_GetNbUsers
 *
 * Parameters:
 *             vlvc_t *p_vlvc : a pointer to the vlvc object
 *
 * Returns:
 *             int : the number of users
 *
 * Description:
 *             Callback used by the GUI to get the number of users.
 *****************************************************************************/
int vlvc_GetNbUsers( vlvc_t *p_vlvc )
{
    return( p_vlvc->i_nb_users );
}

/*****************************************************************************
 * Name: vlvc_GetUserName
 *
 * Parameters:
 *             vlvc_t *p_vlvc        : a pointer to the vlvc object
 *             int    i_user_id      : the user ID
 *             char   *psz_user_name : buffer to store the user name
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Callback used by the GUI to get the specified user's name
 *****************************************************************************/
int vlvc_GetUserName( vlvc_t *p_vlvc, int i_user_id , char *psz_user_name)
{
    if( p_vlvc == NULL || i_user_id < 0 || i_user_id >= p_vlvc->i_nb_users )
        return( VLVC_EARGS );

    memcpy( psz_user_name, p_vlvc->p_users[i_user_id].psz_name, VLVC_MAX_USER_NAME_LENGTH );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_GetUserDesc
 *
 * Parameters:
 *             vlvc_t *p_vlvc        : a pointer to the vlvc object
 *             int    i_user_id      : the user ID
 *             char   *psz_user_desc : the user description
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Callback used by the GUI to get the specified user's description
 *****************************************************************************/
int vlvc_GetUserDesc( vlvc_t *p_vlvc, int i_user_id , char *psz_user_desc)
{
    if( p_vlvc == NULL || i_user_id < 0 || i_user_id >= p_vlvc->i_nb_users )
        return( VLVC_EARGS );

    memcpy( psz_user_desc, p_vlvc->p_users[i_user_id].psz_description, VLVC_MAX_USER_DESC_LENGTH );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_PlayUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : the user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Callback used by the GUI to play the specified user
 *****************************************************************************/
int vlvc_PlayUser( vlvc_t *p_vlvc, int i_user_id )
{
    int         i_count;
    int         i_flag = 0;

    if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CHAT_ROOM ||
        p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_CONFERENCE ||
        p_vlvc->i_self_index == p_vlvc->i_server_index )
    {
        if( p_vlvc == NULL || i_user_id < 0 || i_user_id >= p_vlvc->i_nb_users )
            return( VLVC_EARGS );
        HandleVlmUserControl( p_vlvc, i_user_id,
                              VLVC_PLAY_VLM_CONTROL_COMMAND, NULL );
    }
    else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
    {
        if( p_vlvc == NULL || i_user_id < 0 || i_user_id >= p_vlvc->i_nb_users )
            return( VLVC_EARGS );
        for( i_count = 0; i_count < VLVC_MAX_USERS; i_count++ )
            if ( i_user_id == p_vlvc->p_meeting_auth_users[i_count] ||
                i_user_id == p_vlvc->i_server_index ||
                i_user_id == p_vlvc->i_self_index )
                i_flag = 1;
        if ( !i_flag )
            return( VLC_SUCCESS );
        HandleVlmUserControl( p_vlvc, i_user_id,
                              VLVC_PLAY_VLM_CONTROL_COMMAND, NULL );
    }
    else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
    {
        if( p_vlvc == NULL || i_user_id < 0 || i_user_id >= p_vlvc->i_nb_users )
            return( VLVC_EARGS );
        if( i_user_id != p_vlvc->i_amphi_auth_user &&
            i_user_id != p_vlvc->i_self_index &&
            i_user_id != p_vlvc->i_server_index )
            return( VLC_SUCCESS );
        HandleVlmUserControl( p_vlvc, i_user_id,
                              VLVC_PLAY_VLM_CONTROL_COMMAND, NULL );
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_StopUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : the user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Callback used by the GUI to stop the specified user
 *****************************************************************************/
int vlvc_StopUser( vlvc_t *p_vlvc, int i_user_id )
{
    if( p_vlvc == NULL || i_user_id < 0 || i_user_id >= p_vlvc->i_nb_users )
        return( VLVC_EARGS );
    HandleVlmUserControl( p_vlvc, i_user_id,
                          VLVC_STOP_VLM_CONTROL_COMMAND, NULL );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_HandUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : the user ID
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Callback used by the GUI to raise hand
 ******************************************************************************/
int vlvc_HandUser( vlvc_t *p_vlvc, int i_user_id )
{
    char                          c_command = VLVC_NET_COMMAND_HAND;

    if( ClientNetWrite( p_vlvc, &c_command,
                        sizeof( c_command ) ) == VLVC_WRITE_ERROR )
        return( VLC_EGENERIC );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_AuthUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : the user ID who will be authorized
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Callback used by the GUI to authorize a user
 ******************************************************************************/
int vlvc_AuthUser( vlvc_t *p_vlvc, int i_user_id )
{
    int                           i_dest_user;
    char                          c_command = VLVC_NET_COMMAND_AUTHORIZE;
    int                           i_test_auth;

    if ( i_user_id != p_vlvc->i_self_index )
    {
        for( i_dest_user = 0; i_dest_user < p_vlvc->i_nb_users; i_dest_user++ )
            if( i_dest_user != p_vlvc->i_self_index )
            {
                ServerNetWrite( p_vlvc, i_dest_user, &c_command, sizeof( c_command ) );
                ServerNetWrite( p_vlvc, i_dest_user, &i_user_id, sizeof( i_user_id ) );
            }
        if ( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
        {
            i_test_auth = 0;
            for( i_dest_user = 0; i_dest_user < VLVC_MAX_USERS &&
                     !i_test_auth; i_dest_user++ )
                     if ( p_vlvc->p_meeting_auth_users[i_dest_user] == i_user_id )
                         i_test_auth++;
            if ( !i_test_auth )
            {
                for( i_dest_user = 0; i_dest_user < VLVC_MAX_USERS &&
                         !i_test_auth; i_dest_user++ )
                    if ( p_vlvc->p_meeting_auth_users[i_dest_user] == VLVC_AUTH_FREE )
                    {
                        i_test_auth++;
                        p_vlvc->p_meeting_auth_users[i_dest_user] = i_user_id;
                    }
                vlvc_PlayUser( p_vlvc, i_user_id );
            }
        }
        else if ( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
        {
            vlvc_StopUser( p_vlvc, p_vlvc->i_amphi_auth_user );
            p_vlvc->i_amphi_auth_user = i_user_id;
            vlvc_PlayUser( p_vlvc, i_user_id );
        }
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_UnauthUser
 *
 * Parameters:
 *             vlvc_t *p_vlvc   : a pointer to the vlvc object
 *             int    i_user_id : the user ID who will be unauthorized
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Callback used by the GUI to unauthorize a user
 ******************************************************************************/
int vlvc_UnauthUser( vlvc_t *p_vlvc, int i_user_id )
{
    int                           i_dest_user;
    char                          c_command = VLVC_NET_COMMAND_UNAUTHORIZE;

    if( i_user_id != p_vlvc->i_self_index )
    {
        for( i_dest_user = 0; i_dest_user < p_vlvc->i_nb_users; i_dest_user++ )
            if( i_dest_user != p_vlvc->i_self_index )
            {
                ServerNetWrite( p_vlvc, i_dest_user, &c_command, sizeof( c_command ) );
                ServerNetWrite( p_vlvc, i_dest_user, &i_user_id, sizeof( i_user_id ) );
            }
        if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
        {
            for( i_dest_user = 0; i_dest_user < VLVC_MAX_USERS; i_dest_user++ )
                if ( p_vlvc->p_meeting_auth_users[i_dest_user] == i_user_id )
                    p_vlvc->p_meeting_auth_users[i_dest_user] = VLVC_AUTH_FREE;
        }
        else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
            if( p_vlvc->i_amphi_auth_user == i_user_id )
                p_vlvc->i_amphi_auth_user = VLVC_AUTH_FREE;
    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_GetUserAuthState
 *
 * Parameters:
 *             vlvc_t *p_vlvc       : a pointer to the vlvc object
 *             int    i_user_id     : the user ID
 *             int    *i_auth_state : the auth state of the user
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Callback used by the GUI to get the user authorization state
 *****************************************************************************/
int vlvc_GetUserAuthState( vlvc_t *p_vlvc, int i_user_id , int *i_auth_state)
{
    int         i_dest_user;

    *i_auth_state = 0;
    if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_MEETING )
    {
        for( i_dest_user = 0; i_dest_user < VLVC_MAX_USERS; i_dest_user++ )
            if ( p_vlvc->p_meeting_auth_users[i_dest_user] == i_user_id )
                *i_auth_state = 1;
    }
    else if( p_vlvc->p_parameters->i_conference_mode == VLVC_CONFERENCE_MODE_AMPHITHEATRE )
        if( p_vlvc->i_amphi_auth_user == i_user_id )
            *i_auth_state = 1;
    return ( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_SetCallbackChangeNbUsers
 *
 * Parameters:
 *             vlvc_t   *p_vlvc         : a pointer to the vlvc object
 *             void (*) ( int, void * ) : pointer to the callback function
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the GUI to provide a callback used to notify it when
 *             a user joins or leaves.
 *****************************************************************************/
int vlvc_SetCallbackChangeNbUsers( vlvc_t *p_vlvc, void (*NbClientChange) ( int, void * ))
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    p_vlvc->NbClientChange = NbClientChange;
    p_vlvc->b_GUIPresent = VLC_TRUE;
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_SetCallbackStop
 *
 * Parameters:
 *             vlvc_t   *p_vlvc    : a pointer to the vlvc object
 *             void (*) ( void * ) : pointer to the callback function
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the GUI to provide a callback used to stop it.
 *****************************************************************************/
int vlvc_SetCallbackStop( vlvc_t *p_vlvc, void (*Stop) ( void * ))
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    p_vlvc->Stop = Stop;
    p_vlvc->b_GUIPresent = VLC_TRUE;
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_SetCallbackCoreModuleMessage
 *
 * Parameters:
 *             vlvc_t   *p_vlvc                : a pointer to the vlvc object
 *             void (*) ( char *, int, void * ): pointer to the callback
 *                                               function
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the GUI to provide a callback used to display a
 *             graphical message for the user (error, information...)
 *****************************************************************************/
int vlvc_SetCallbackCoreModuleMessage( vlvc_t *p_vlvc, void (*CoreModuleMessage) ( char *, int, void * ) )
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    p_vlvc->CoreModuleMessage = CoreModuleMessage;
    p_vlvc->b_GUIPresent = VLC_TRUE;
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_SetCallChatMessage
 *
 * Parameters:
 *             vlvc_t   *p_vlvc : a pointer to the vlvc object
 *             void (*) ( int, const char *, int, void *  ): pointer to the
 *                                                           callback function
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the GUI to provide a callback used to notify of a new
 *             chat message.
 *****************************************************************************/
int vlvc_SetCallbackChatMessage( vlvc_t *p_vlvc, void ( *ChatMessage ) ( int, const char *, int, void *  ) )
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    p_vlvc->ChatMessage = ChatMessage;
    p_vlvc->b_GUIPresent = VLC_TRUE;
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_SetCallbackDisableHandBtn
 *
 * Parameters:
 *             vlvc_t   *p_vlvc : a pointer to the vlvc object
 *             void (*) ( int, const char *, int, void *  ): pointer to the
 *                                                           callback function
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the GUI to provide a callback used to disable the
 *             "Raise Hand" button
 *****************************************************************************/
int vlvc_SetCallbackDisableHandBtn( vlvc_t *p_vlvc, void ( *DisableHandBtn ) )
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    p_vlvc->DisableHandBtn = DisableHandBtn;
    p_vlvc->b_GUIPresent = VLC_TRUE;
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_SetCallbackEnableAuthBtn
 *
 * Parameters:
 *             vlvc_t   *p_vlvc : a pointer to the vlvc object
 *             void (*) ( int, const char *, int, void *  ): pointer to the
 *                                                           callback function
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the GUI to provide a callback used to enable the
 *             "Authorize" button
 *****************************************************************************/
int vlvc_SetCallbackEnableAuthBtn( vlvc_t *p_vlvc, void ( *EnableAuthBtn ) )
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    p_vlvc->EnableAuthBtn = EnableAuthBtn;
    p_vlvc->b_GUIPresent = VLC_TRUE;
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_SetCallbackEnableUnauthBtn
 *
 * Parameters:
 *             vlvc_t   *p_vlvc : a pointer to the vlvc object
 *             void (*) ( int, const char *, int, void *  ): pointer to the
 *                                                           callback function
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the GUI to provide a callback used to enable the
 *             "Unauthorize" button
 *****************************************************************************/
int vlvc_SetCallbackEnableUnauthBtn( vlvc_t *p_vlvc, void ( *EnableUnauthBtn ) )
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    p_vlvc->EnableUnauthBtn = EnableUnauthBtn;
    p_vlvc->b_GUIPresent = VLC_TRUE;
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_SetCallbackAuthUser
 *
 * Parameters:
 *             vlvc_t   *p_vlvc         : a pointer to the vlvc object
 *             void (*) ( int, void * ) : pointer to the callback function
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the GUI to provide a callback used to notify it when
 *             a user is authorized
 *****************************************************************************/
int vlvc_SetCallbackAuthUser( vlvc_t *p_vlvc, void (*AuthUser) ( int, void * ))
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    p_vlvc->AuthUser = AuthUser;
    p_vlvc->b_GUIPresent = VLC_TRUE;
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: vlvc_SetCallbackUnauthUser
 *
 * Parameters:
 *             vlvc_t   *p_vlvc         : a pointer to the vlvc object
 *             void (*) ( int, void * ) : pointer to the callback function
 *
 * Returns:
 *             int : VLVC return code
 *
 * Description:
 *             Called by the GUI to provide a callback used to notify it when
 *             a user is unauthorized
 *****************************************************************************/
int vlvc_SetCallbackUnauthUser( vlvc_t *p_vlvc, void (*UnauthUser) ( int, void * ))
{
    if( p_vlvc == NULL )
        return( VLVC_EARGS );
    p_vlvc->UnauthUser = UnauthUser;
    p_vlvc->b_GUIPresent = VLC_TRUE;
    return( VLC_SUCCESS );
}
